@@ -5,13 +5,21 @@
  * Based on Sound-Juicer's ripping code
  *
  * Copyright (C) 2003 Ross Burton <ross@burtonini.com>
- * Copyright (C) 2006 James Livingston <jrl@ids.org.au>
+ * Copyright (C) 2006 James Livingston <doclivingston@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
+ * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -62,6 +70,7 @@ struct _RBEncoderGstPrivate {
 	GstFormat position_format;
 	gint64 total_length;
 	guint progress_id;
+	char *dest_uri;
 };
 
 G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
@@ -116,6 +125,10 @@ rb_encoder_gst_class_init (RBEncoderGstClass *klass)
 	caps = gst_caps_new_simple ("audio/x-vorbis",
 				    NULL);
 	g_hash_table_insert (klass->mime_caps_table, "application/ogg", caps);
+
+	/* FLAC */
+	caps = gst_caps_new_simple ("audio/x-flac", NULL);
+	g_hash_table_insert (klass->mime_caps_table, "audio/flac", caps);
 }
 
 static void
@@ -146,6 +159,8 @@ rb_encoder_gst_finalize (GObject *object)
 		encoder->priv->pipeline = NULL;
 	}
 
+	g_free (encoder->priv->dest_uri);
+
         G_OBJECT_CLASS (rb_encoder_gst_parent_class)->finalize (object);
 }
 
@@ -166,6 +181,9 @@ static void
 rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 {
 	GError *error = NULL;
+	guint64 dest_size;
+	GnomeVFSFileInfo *file_info;
+	GnomeVFSResult result;
 
 	g_return_if_fail (encoder->priv->completion_emitted == FALSE);
 
@@ -187,8 +205,22 @@ rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 		g_error_free (error);
 	}
 
+	/* find the size of the output file, assuming we can get at it with gnome-vfs */
+	dest_size = 0;
+	file_info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info (encoder->priv->dest_uri, file_info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (result == GNOME_VFS_OK && (file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)) {
+		dest_size = file_info->size;
+		rb_debug ("destination file size: %" G_GUINT64_FORMAT, dest_size);
+	} else {
+		rb_debug ("couldn't get size of destination %s: %s",
+			  encoder->priv->dest_uri,
+			  gnome_vfs_result_to_string (result));
+	}
+	gnome_vfs_file_info_unref (file_info);
+
 	encoder->priv->completion_emitted = TRUE;
-	_rb_encoder_emit_completed (RB_ENCODER (encoder));
+	_rb_encoder_emit_completed (RB_ENCODER (encoder), dest_size);
 }
 
 static gboolean
@@ -410,9 +442,9 @@ add_tags_from_entry (RBEncoderGst *encoder,
 				  GST_TAG_MUSICBRAINZ_ALBUMARTISTID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID),
 				  NULL);
 	}
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME)) {
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME)) {
 		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_SORTNAME, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME),
+				  GST_TAG_MUSICBRAINZ_SORTNAME, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME),
 				  NULL);
 	}
 #endif
@@ -981,6 +1013,8 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 			entry_mime_type = "audio/mpeg";
 	} else if (rb_safe_strcmp (entry_mime_type, "application/x-id3") == 0) {
 		entry_mime_type = "audio/mpeg";
+	} else if (rb_safe_strcmp (entry_mime_type, "audio/x-flac") == 0) {
+		entry_mime_type = "audio/flac";
 	}
 
 	vfsresult = create_parent_dirs (dest);
@@ -990,7 +1024,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 					     gnome_vfs_result_to_string (vfsresult));
 
 		_rb_encoder_emit_error (encoder, error);
-		_rb_encoder_emit_completed (encoder);
+		_rb_encoder_emit_completed (encoder, 0);
 		g_error_free (error);
 		return FALSE;
 	}
@@ -1014,6 +1048,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		}
 	}
 
+	priv->dest_uri = g_strdup (dest);
 	if (copy) {
 		priv->total_length = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
 		priv->position_format = GST_FORMAT_BYTES;
@@ -1035,12 +1070,13 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 
 		rb_encoder_gst_emit_error (enc, error);
 		g_error_free (error);
-		if (enc->priv->pipeline == NULL)
+		if (enc->priv->pipeline == NULL) {
 			rb_encoder_gst_emit_completed (enc);
-		else
+		} else {
 			/* this will unref the pipeline and call emit_completed
 			 */
 			rb_encoder_gst_cancel (encoder);
+		}
 	}
 
 	return result;
