@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -35,6 +43,7 @@
 
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
+#include "rb-util.h"
 
 static GHashTable *files = NULL;
 
@@ -652,6 +661,7 @@ has_valid_scheme (const char *uri)
 
 /**
  * eel_make_uri_from_shell_arg:
+ * @location: a possibly mangled "uri"
  *
  * Similar to eel_make_uri_from_input, except that:
  * 
@@ -659,10 +669,7 @@ has_valid_scheme (const char *uri)
  * 2) doesn't bother stripping leading/trailing white space
  * 3) doesn't bother with ~ expansion--that's done by the shell
  *
- * @location: a possibly mangled "uri"
- *
- * returns a newly allocated uri
- *
+ * Return value: a newly allocated uri
  **/
 char *
 rb_uri_resolve_relative (const char *location)
@@ -774,6 +781,65 @@ rb_uri_is_hidden (const char *text_uri)
 	return g_utf8_strrchr (text_uri, -1, GNOME_VFS_URI_PATH_CHR)[1] == '.';
 }
 
+gboolean
+rb_uri_could_be_podcast (const char *uri, gboolean *is_opml)
+{
+	const char *query_string;
+
+	if (is_opml != NULL)
+		*is_opml = FALSE;
+
+	/* Check the scheme is a possible one first */
+	if (g_str_has_prefix (uri, "http") == FALSE &&
+	    g_str_has_prefix (uri, "itpc:") == FALSE &&
+	    g_str_has_prefix (uri, "itms:") == FALSE) {
+	    	rb_debug ("'%s' can't be a Podcast or OPML file, not the right scheme", uri);
+	    	return FALSE;
+	}
+
+	/* Now, check whether the iTunes Music Store link
+	 * is a podcast */
+	if (g_str_has_prefix (uri, "itms:") != FALSE
+	    && strstr (uri, "phobos.apple.com") != NULL
+	    && strstr (uri, "viewPodcast") != NULL)
+		return TRUE;
+
+	query_string = strchr (uri, '?');
+	if (query_string == NULL) {
+		query_string = uri + strlen (uri);
+	}
+
+	/* FIXME hacks */
+	if (strstr (uri, "rss") != NULL ||
+	    strstr (uri, "atom") != NULL ||
+	    strstr (uri, "feed") != NULL) {
+	    	rb_debug ("'%s' should be Podcast file, HACK", uri);
+	    	return TRUE;
+	} else if (strstr (uri, "opml") != NULL) {
+		rb_debug ("'%s' should be an OPML file, HACK", uri);
+		if (is_opml != NULL)
+			*is_opml = TRUE;
+		return TRUE;
+	}
+
+	if (strncmp (query_string - 4, ".rss", 4) == 0 ||
+	    strncmp (query_string - 4, ".xml", 4) == 0 ||
+	    strncmp (query_string - 5, ".atom", 5) == 0 ||
+	    strncmp (uri, "itpc", 4) == 0 ||
+	    (strstr (uri, "phobos.apple.com/") != NULL && strstr (uri, "viewPodcast") != NULL) ||
+	    strstr (uri, "itunes.com/podcast") != NULL) {
+	    	rb_debug ("'%s' should be Podcast file", uri);
+	    	return TRUE;
+	} else if (strncmp (query_string - 5, ".opml", 5) == 0) {
+		rb_debug ("'%s' should be an OPML file", uri);
+		if (is_opml != NULL)
+			*is_opml = TRUE;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 char *
 rb_uri_make_hidden (const char *text_uri)
 {
@@ -1204,3 +1270,46 @@ rb_uri_get_short_path_name (const char *uri)
 	}
 }
 
+char *
+rb_sanitize_uri_for_filesystem(gchar *uri)
+{
+	gchar *filesystem = rb_uri_get_filesystem_type(uri);
+	gchar *sane_uri = NULL;
+
+	if (!filesystem)
+		return g_strdup(uri);
+
+	if (!strcmp(filesystem, "fat") ||
+	    !strcmp(filesystem, "vfat") ) {
+	    	gchar *hostname = NULL;
+		GError *error = NULL;
+	    	gchar *full_path = g_filename_from_uri(uri, &hostname, &error);
+
+		if (error) {
+			g_error_free(error);
+			g_free(filesystem);
+			g_free(full_path);
+			return g_strdup(uri);
+		}
+
+		g_strdelimit (full_path, "\"", '\'');
+		g_strdelimit (full_path, ":|<>*?\\", '_');
+
+		/* create a new uri from this */
+		sane_uri = g_filename_to_uri(full_path, hostname, &error);
+
+		g_free(hostname);
+		g_free(full_path);
+
+		if (error) {
+			g_error_free(error);
+			g_free(filesystem);
+			return g_strdup(uri);
+		}
+	}
+
+	/* add workarounds for other filesystems limitations here */
+
+	g_free(filesystem);
+	return sane_uri ? sane_uri : g_strdup(uri);
+}
