@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -71,6 +79,8 @@ char*		rb_uri_append_uri	(const char *uri,
 void		rb_file_helpers_init	(void);
 void		rb_file_helpers_shutdown(void);
 
+char *		rb_sanitize_uri_for_filesystem(gchar *uri);
+
 G_END_DECLS
 
 #endif /* __RB_FILE_HELPERS_H */
