@@ -100,7 +100,52 @@ rb_marshal_BOOLEAN__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:POINTER (./rb-marshal.list:2) */
+/* BOOLEAN:BOXED (./rb-marshal.list:2) */
+extern void rb_marshal_BOOLEAN__BOXED (GClosure     *closure,
+                                       GValue       *return_value,
+                                       guint         n_param_values,
+                                       const GValue *param_values,
+                                       gpointer      invocation_hint,
+                                       gpointer      marshal_data);
+void
+rb_marshal_BOOLEAN__BOXED (GClosure     *closure,
+                           GValue       *return_value G_GNUC_UNUSED,
+                           guint         n_param_values,
+                           const GValue *param_values,
+                           gpointer      invocation_hint G_GNUC_UNUSED,
+                           gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__BOXED) (gpointer     data1,
+                                                   gpointer     arg_1,
+                                                   gpointer     data2);
+  register GMarshalFunc_BOOLEAN__BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_boxed (param_values + 1),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* BOOLEAN:POINTER (./rb-marshal.list:3) */
 extern void rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -145,7 +190,7 @@ rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:POINTER,POINTER,POINTER (./rb-marshal.list:3) */
+/* BOOLEAN:POINTER,POINTER,POINTER (./rb-marshal.list:4) */
 extern void rb_marshal_BOOLEAN__POINTER_POINTER_POINTER (GClosure     *closure,
                                                          GValue       *return_value,
                                                          guint         n_param_values,
@@ -194,7 +239,54 @@ rb_marshal_BOOLEAN__POINTER_POINTER_POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,STRING,OBJECT (./rb-marshal.list:4) */
+/* BOOLEAN:STRING,BOOLEAN (./rb-marshal.list:5) */
+extern void rb_marshal_BOOLEAN__STRING_BOOLEAN (GClosure     *closure,
+                                                GValue       *return_value,
+                                                guint         n_param_values,
+                                                const GValue *param_values,
+                                                gpointer      invocation_hint,
+                                                gpointer      marshal_data);
+void
+rb_marshal_BOOLEAN__STRING_BOOLEAN (GClosure     *closure,
+                                    GValue       *return_value G_GNUC_UNUSED,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint G_GNUC_UNUSED,
+                                    gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__STRING_BOOLEAN) (gpointer     data1,
+                                                            gpointer     arg_1,
+                                                            gboolean     arg_2,
+                                                            gpointer     data2);
+  register GMarshalFunc_BOOLEAN__STRING_BOOLEAN callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_string (param_values + 1),
+                       g_marshal_value_peek_boolean (param_values + 2),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* BOOLEAN:STRING,STRING,OBJECT (./rb-marshal.list:6) */
 extern void rb_marshal_BOOLEAN__STRING_STRING_OBJECT (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -243,7 +335,7 @@ rb_marshal_BOOLEAN__STRING_STRING_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* INT:VOID (./rb-marshal.list:5) */
+/* INT:VOID (./rb-marshal.list:7) */
 extern void rb_marshal_INT__VOID (GClosure     *closure,
                                   GValue       *return_value,
                                   guint         n_param_values,
@@ -286,7 +378,7 @@ rb_marshal_INT__VOID (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* OBJECT:OBJECT (./rb-marshal.list:6) */
+/* OBJECT:OBJECT (./rb-marshal.list:8) */
 extern void rb_marshal_OBJECT__OBJECT (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -331,7 +423,7 @@ rb_marshal_OBJECT__OBJECT (GClosure     *closure,
   g_value_take_object (return_value, v_return);
 }
 
-/* STRING:STRING (./rb-marshal.list:7) */
+/* STRING:STRING (./rb-marshal.list:9) */
 extern void rb_marshal_STRING__STRING (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -376,7 +468,7 @@ rb_marshal_STRING__STRING (GClosure     *closure,
   g_value_take_string (return_value, v_return);
 }
 
-/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:8) */
+/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:10) */
 extern void rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -419,7 +511,7 @@ rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* BOXED:BOXED (./rb-marshal.list:9) */
+/* BOXED:BOXED (./rb-marshal.list:11) */
 extern void rb_marshal_BOXED__BOXED (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -464,7 +556,7 @@ rb_marshal_BOXED__BOXED (GClosure     *closure,
   g_value_take_boxed (return_value, v_return);
 }
 
-/* VOID:BOXED,BOXED (./rb-marshal.list:10) */
+/* VOID:BOXED,BOXED (./rb-marshal.list:12) */
 extern void rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -507,7 +599,7 @@ rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:11) */
+/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:13) */
 extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -554,7 +646,7 @@ rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,OBJECT (./rb-marshal.list:12) */
+/* VOID:BOXED,OBJECT (./rb-marshal.list:14) */
 extern void rb_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -597,7 +689,7 @@ rb_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,POINTER (./rb-marshal.list:13) */
+/* VOID:BOXED,POINTER (./rb-marshal.list:15) */
 extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -640,7 +732,7 @@ rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:14) */
+/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:16) */
 extern void rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -685,7 +777,7 @@ rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,ULONG (./rb-marshal.list:15) */
+/* VOID:BOXED,ULONG (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -728,7 +820,7 @@ rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:DOUBLE,LONG (./rb-marshal.list:16) */
+/* VOID:DOUBLE,LONG (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -771,7 +863,48 @@ rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT (./rb-marshal.list:17) */
+/* VOID:UINT64 (./rb-marshal.list:19) */
+extern void rb_marshal_VOID__UINT64 (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
+void
+rb_marshal_VOID__UINT64 (GClosure     *closure,
+                         GValue       *return_value G_GNUC_UNUSED,
+                         guint         n_param_values,
+                         const GValue *param_values,
+                         gpointer      invocation_hint G_GNUC_UNUSED,
+                         gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT64) (gpointer     data1,
+                                             guint64      arg_1,
+                                             gpointer     data2);
+  register GMarshalFunc_VOID__UINT64 callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT64) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint64 (param_values + 1),
+            data2);
+}
+
+/* VOID:INT,INT (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__INT_INT (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
@@ -814,7 +947,7 @@ rb_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:18) */
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -859,7 +992,7 @@ rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:19) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -904,7 +1037,7 @@ rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:20) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -955,7 +1088,7 @@ rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:21) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -1000,7 +1133,7 @@ rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./rb-marshal.list:22) */
+/* VOID:POINTER,INT (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1043,7 +1176,7 @@ rb_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:23) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -1088,7 +1221,7 @@ rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,LONG,LONG (./rb-marshal.list:24) */
+/* VOID:POINTER,LONG,LONG (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -1133,7 +1266,7 @@ rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:25) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -1176,7 +1309,7 @@ rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:26) */
+/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -1221,7 +1354,7 @@ rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (./rb-marshal.list:27) */
+/* VOID:POINTER,UINT (./rb-marshal.list:30) */
 extern void rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -1264,7 +1397,7 @@ rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:28) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1307,7 +1440,7 @@ rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:29) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:32) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1350,7 +1483,7 @@ rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./rb-marshal.list:30) */
+/* VOID:STRING,INT (./rb-marshal.list:33) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -1393,7 +1526,7 @@ rb_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,POINTER (./rb-marshal.list:31) */
+/* VOID:STRING,POINTER (./rb-marshal.list:34) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1436,7 +1569,7 @@ rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./rb-marshal.list:32) */
+/* VOID:STRING,STRING (./rb-marshal.list:35) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1479,7 +1612,7 @@ rb_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:33) */
+/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:36) */
 extern void rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1524,7 +1657,7 @@ rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:34) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:37) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1569,7 +1702,7 @@ rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:35) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:38) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -1618,7 +1751,7 @@ rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:36) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:39) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1661,7 +1794,7 @@ rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:37) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:40) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1704,7 +1837,7 @@ rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:38) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:41) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
