@@ -11,6 +11,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -24,6 +32,8 @@
 
 #define __EXTENSIONS__
 
+#include "config.h"
+
 #include <errno.h>
 
 #include <string.h>
@@ -35,10 +45,9 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-value.h>
 
+#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
-#include <libsoup/soup-uri.h>
 
-#include "config.h"
 #include "eel-gconf-extensions.h"
 #include "rb-audioscrobbler.h"
 #include "rb-debug.h"
@@ -52,7 +61,9 @@
 #include "rb-proxy-config.h"
 #include "rb-cut-and-paste-code.h"
 #include "rb-plugin.h"
+#include "rb-util.h"
 
+#include "rb-audioscrobbler-entry.h"
 
 #define CLIENT_ID "rbx"
 #define CLIENT_VERSION VERSION
@@ -61,30 +72,7 @@
 #define SCROBBLER_URL "http://post.audioscrobbler.com/"
 #define SCROBBLER_VERSION "1.1"
 
-#define SCROBBLER_DATE_FORMAT "%Y%%2D%m%%2D%d%%20%H%%3A%M%%3A%S"
-
-#define EXTRA_URI_ENCODE_CHARS	"&+"
-
-typedef struct
-{
-	gchar *artist;
-	gchar *album;
-	gchar *title;
-	guint length;
-	gchar *mbid;
-	time_t play_time;
-} AudioscrobblerEntry;
-
-typedef struct
-{
-	gchar *artist;
-	gchar *album;
-	gchar *title;
-	guint length;
-	gchar *mbid;
-	gchar *timestamp;
-} AudioscrobblerEncodedEntry;
-
+#define USER_AGENT	"Rhythmbox/" VERSION
 
 struct _RBAudioscrobblerPrivate
 {
@@ -163,13 +151,6 @@ struct _RBAudioscrobblerPrivate
 #define RB_AUDIOSCROBBLER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOSCROBBLER, RBAudioscrobblerPrivate))
 
 
-static void	     audioscrobbler_entry_init (AudioscrobblerEntry *entry);
-static void	     audioscrobbler_entry_free (AudioscrobblerEntry *entry);
-static void          audioscrobbler_encoded_entry_free (AudioscrobblerEncodedEntry *entry);
-static AudioscrobblerEncodedEntry *audioscrobbler_entry_encode (AudioscrobblerEntry *entry);
-
-
-
 static gboolean	     rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler);
 static int	     rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_print_queue (RBAudioscrobbler *audioscrobbler, gboolean submission);
@@ -193,14 +174,20 @@ static gboolean	     rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobb
 
 static gchar *	     mkmd5 (char *string);
 static void	     rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage *msg);
-static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
-					       char *url,
-					       char *post_data,
-						   SoupMessageCallbackFn response_handler);
+
 static void	     rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler);
-static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
 static void	     rb_audioscrobbler_submit_queue (RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
+						char *url,
+						char *post_data,
+						SoupSessionCallback response_handler);
+#if defined(HAVE_LIBSOUP_2_4)
+static void	     rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
+static void	     rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
+#else
+static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
 static void	     rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data);
+#endif
 
 static void	     rb_audioscrobbler_import_settings (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler);
@@ -407,7 +394,7 @@ rb_audioscrobbler_finalize (GObject *object)
 	g_free (audioscrobbler->priv->password);
 	g_free (audioscrobbler->priv->submit_url);
 	if (audioscrobbler->priv->currently_playing != NULL) {
-		audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
+		rb_audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
 		audioscrobbler->priv->currently_playing = NULL;
 	}
 
@@ -612,7 +599,7 @@ maybe_add_current_song_to_queue (RBAudioscrobbler *audioscrobbler)
 			/* not sure about this - what if I skip to somewhere towards
 			 * the end, but then go back and listen to the whole song?
 			 */
-			audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
+			rb_audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
 			audioscrobbler->priv->currently_playing = NULL;
 
 		}
@@ -688,18 +675,31 @@ mkmd5 (char *string)
 static void
 rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage *msg)
 {
+	gboolean successful;
 	rb_debug ("Parsing response, status=%d", msg->status_code);
-
-	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL) {
-		gchar *body;
+	
+	successful = FALSE;
+#if defined(HAVE_LIBSOUP_2_4)
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && msg->response_body->length != 0)
+		successful = TRUE;
+#else
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL)
+		successful = TRUE;
+#endif
+	if (successful) {
 		gchar **breaks;
+		int i;
+#if defined(HAVE_LIBSOUP_2_2)
+		gchar *body;
 
 		body = g_malloc0 ((msg->response).length + 1);
 		memcpy (body, (msg->response).body, (msg->response).length);
 
 		g_strstrip (body);
 		breaks = g_strsplit (body, "\n", 4);
-		int i;
+#else
+		breaks = g_strsplit (msg->response_body->data, "\n", 4);
+#endif
 
 		g_free (audioscrobbler->priv->status_msg);
 		audioscrobbler->priv->status = STATUS_OK;
@@ -771,10 +771,12 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 			audioscrobbler->priv->submit_next = time(NULL) + audioscrobbler->priv->submit_interval;
 
 		g_strfreev (breaks);
+#if defined(HAVE_LIBSOUP_2_2)
 		g_free (body);
+#endif
 	} else {
 		audioscrobbler->priv->status = REQUEST_FAILED;
-		audioscrobbler->priv->status_msg = g_strdup (soup_status_get_phrase (msg->status_code));
+		audioscrobbler->priv->status_msg = g_strdup (msg->reason_phrase);
 	}
 }
 
@@ -793,24 +795,25 @@ static void
 rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 			   char *url,
 			   char *post_data,
-			   SoupMessageCallbackFn response_handler)
+			   SoupSessionCallback response_handler)
 {
 	SoupMessage *msg;
 
 	msg = soup_message_new (post_data == NULL ? "GET" : "POST", url);
+	soup_message_headers_append (msg->request_headers, "User-Agent", USER_AGENT);
 
 	if (post_data != NULL) {
 		rb_debug ("Submitting to Audioscrobbler: %s", post_data);
 		soup_message_set_request (msg,
 					  "application/x-www-form-urlencoded",
-					  SOUP_BUFFER_SYSTEM_OWNED,
+					  SOUP_MEMORY_TAKE,
 					  post_data,
 					  strlen (post_data));
 	}
 
 	/* create soup session, if we haven't got one yet */
 	if (!audioscrobbler->priv->soup_session) {
-		SoupUri *uri;
+		SoupURI *uri;
 
 		uri = rb_proxy_config_get_libsoup_uri (audioscrobbler->priv->proxy_config);
 		audioscrobbler->priv->soup_session = soup_session_async_new_with_options (
@@ -891,8 +894,13 @@ rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler)
 }
 
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
+#else
 static void
 rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
+#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER(user_data);
 
@@ -993,7 +1001,7 @@ rb_audioscrobbler_build_post_data (RBAudioscrobbler *audioscrobbler,
 		gchar *new;
 		/* remove first queue entry */
 		entry = g_queue_pop_head (audioscrobbler->priv->queue);
-		encoded = audioscrobbler_entry_encode (entry);
+		encoded = rb_audioscrobbler_entry_encode (entry);
 		new = g_strdup_printf ("%sa[%d]=%s&t[%d]=%s&b[%d]=%s&m[%d]=%s&l[%d]=%d&i[%d]=%s&",
 				       post_data,
 				       i, encoded->artist,
@@ -1002,7 +1010,7 @@ rb_audioscrobbler_build_post_data (RBAudioscrobbler *audioscrobbler,
 				       i, encoded->mbid,
 				       i, encoded->length,
 				       i, encoded->timestamp);
-		audioscrobbler_encoded_entry_free (encoded);
+		rb_audioscrobbler_encoded_entry_free (encoded);
 		g_free (post_data);
 		post_data = new;
 
@@ -1049,8 +1057,13 @@ rb_g_queue_concat (GQueue *q1, GQueue *q2)
 	}
 }
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
+#else
 static void
 rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
+#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER (user_data);
 
@@ -1232,7 +1245,7 @@ static void
 rb_audioscrobbler_proxy_config_changed_cb (RBProxyConfig *config,
 					   RBAudioscrobbler *audioscrobbler)
 {
-	SoupUri *uri;
+	SoupURI *uri;
 
 	if (audioscrobbler->priv->soup_session) {
 		uri = rb_proxy_config_get_libsoup_uri (config);
@@ -1252,12 +1265,17 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 {
 	rb_debug ("GConf key updated: \"%s\"", entry->key);
 	if (strcmp (entry->key, CONF_AUDIOSCROBBLER_USERNAME) == 0) {
-		const gchar *username;
+		const char *username;
+
+		username = gconf_value_get_string (entry->value);
+		if (rb_safe_strcmp (username, audioscrobbler->priv->username) == 0) {
+			rb_debug ("username not modified");
+			return;
+		}
 
 		g_free (audioscrobbler->priv->username);
 		audioscrobbler->priv->username = NULL;
 
-		username = gconf_value_get_string (entry->value);
 		if (username != NULL) {
 			audioscrobbler->priv->username = g_strdup (username);
 		}
@@ -1269,13 +1287,19 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 		}
 
 		audioscrobbler->priv->handshake = FALSE;
+		audioscrobbler->priv->handshake_next = 0;
 	} else if (strcmp (entry->key, CONF_AUDIOSCROBBLER_PASSWORD) == 0) {
-		const gchar *password;
+		const char *password;
+
+		password = gconf_value_get_string (entry->value);
+		if (rb_safe_strcmp (password, audioscrobbler->priv->password) == 0) {
+			rb_debug ("password not modified");
+			return;
+		}
 
 		g_free (audioscrobbler->priv->password);
 		audioscrobbler->priv->password = NULL;
 
-		password = gconf_value_get_string (entry->value);
 		if (password != NULL) {
 			audioscrobbler->priv->password = g_strdup (password);
 		}
@@ -1285,6 +1309,9 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 			gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->password_entry),
 					    v ? v : "");
 		}
+
+		audioscrobbler->priv->handshake = FALSE;
+		audioscrobbler->priv->handshake_next = 0;
 	} else {
 		rb_debug ("Unhandled GConf key updated: \"%s\"", entry->key);
 	}
@@ -1299,7 +1326,7 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 	guint time;
 
 	if (audioscrobbler->priv->currently_playing != NULL) {
-		audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
+		rb_audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
 		audioscrobbler->priv->currently_playing = NULL;
 	}
 
@@ -1361,129 +1388,9 @@ rb_audioscrobbler_password_entry_activate_cb (GtkEntry *entry,
 	/* ? */
 }
 
-/* AudioscrobblerEntry functions: */
-static void
-audioscrobbler_entry_init (AudioscrobblerEntry *entry)
-{
-	entry->artist = g_strdup ("");
-	entry->album = g_strdup ("");
-	entry->title = g_strdup ("");
-	entry->length = 0;
-	entry->play_time = 0;
-	entry->mbid = g_strdup ("");
-}
-
-static void
-audioscrobbler_entry_free (AudioscrobblerEntry *entry)
-{
-	g_free (entry->artist);
-	g_free (entry->album);
-	g_free (entry->title);
-	g_free (entry->mbid);
-
-	g_free (entry);
-}
-
-static AudioscrobblerEncodedEntry *
-audioscrobbler_entry_encode (AudioscrobblerEntry *entry)
-{
-
-	AudioscrobblerEncodedEntry *encoded;
-
-	encoded = g_new0 (AudioscrobblerEncodedEntry, 1);
-	
-	encoded->artist = soup_uri_encode (entry->artist, 
-					   EXTRA_URI_ENCODE_CHARS);
-	encoded->title = soup_uri_encode (entry->title,
-					  EXTRA_URI_ENCODE_CHARS);
-	encoded->album = soup_uri_encode (entry->album, 
-					  EXTRA_URI_ENCODE_CHARS);
-	encoded->mbid = soup_uri_encode (entry->mbid, 
-					 EXTRA_URI_ENCODE_CHARS);
-	encoded->timestamp = g_new0 (gchar, 30);
-	strftime (encoded->timestamp, 30, SCROBBLER_DATE_FORMAT, 
-		  gmtime (&entry->play_time));
-
-	encoded->length = entry->length;
-
-	return encoded;
-}
-
-static 
-void audioscrobbler_encoded_entry_free (AudioscrobblerEncodedEntry *entry)
-{
-	g_free (entry->artist);
-	g_free (entry->album);
-	g_free (entry->title);
-	g_free (entry->mbid);
-	g_free (entry->timestamp);
-
-	g_free (entry);
-}
 
 
 /* Queue functions: */
-
-static AudioscrobblerEntry*
-rb_audioscrobbler_load_entry_from_string (const char *string)
-{
-	AudioscrobblerEntry *entry;
-	int i = 0;
-	char **breaks;
-
-	entry = g_new0 (AudioscrobblerEntry, 1);
-	audioscrobbler_entry_init (entry);
-
-	breaks = g_strsplit (string, "&", 6);
-
-	for (i = 0; breaks[i] != NULL; i++) {
-		char **breaks2 = g_strsplit (breaks[i], "=", 2);
-
-		if (breaks2[0] != NULL && breaks2[1] != NULL) {
-			if (g_str_has_prefix (breaks2[0], "a")) {
-				g_free (entry->artist);
-				entry->artist = g_strdup (breaks2[1]);
-				soup_uri_decode (entry->artist);
-			}
-			if (g_str_has_prefix (breaks2[0], "t")) {
-				g_free (entry->title);
-				entry->title = g_strdup (breaks2[1]);
-				soup_uri_decode (entry->title);
-			}
-			if (g_str_has_prefix (breaks2[0], "b")) {
-				g_free (entry->album);
-				entry->album = g_strdup (breaks2[1]);
-				soup_uri_decode (entry->album);
-			}
-			if (g_str_has_prefix (breaks2[0], "m")) {
-				g_free (entry->mbid);
-				entry->mbid = g_strdup (breaks2[1]);
-				soup_uri_decode (entry->mbid);
-			}
-			if (g_str_has_prefix (breaks2[0], "l")) {
-				entry->length = atoi (breaks2[1]);
-			}
-			if (g_str_has_prefix (breaks2[0], "i")) {
-				struct tm tm;
-				strptime (breaks2[1], SCROBBLER_DATE_FORMAT, 
-					  &tm);
-				entry->play_time = mktime (&tm);
-			}
-		}
-
-		g_strfreev (breaks2);
-	}
-
-	g_strfreev (breaks);
-
-	if (strcmp (entry->artist, "") == 0 || strcmp (entry->title, "") == 0) {
-		audioscrobbler_entry_free (entry);
-		entry = NULL;
-	}
-
-	return entry;
-}
-
 static gboolean
 rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 {
@@ -1514,7 +1421,7 @@ rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 				break;
 			*end = 0;
 
-			entry = rb_audioscrobbler_load_entry_from_string (start);
+			entry = rb_audioscrobbler_entry_load_from_string (start);
 			if (entry) {
 				g_queue_push_tail (audioscrobbler->priv->queue,
 						   entry);
@@ -1534,23 +1441,6 @@ rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 	return (result == GNOME_VFS_OK);
 }
 
-static char *
-rb_audioscrobbler_save_entry_to_string (AudioscrobblerEntry *entry)
-{
-	char *result;
-	AudioscrobblerEncodedEntry *encoded;
-
-	encoded = audioscrobbler_entry_encode (entry);
-	result = g_strdup_printf ("a=%s&t=%s&b=%s&m=%s&l=%d&i=%s\n",
-				  encoded->artist, 
-				  encoded->title,
-				  encoded->album, 
-				  encoded->mbid,
-				  encoded->length,
-				  encoded->timestamp);
-	audioscrobbler_encoded_entry_free (encoded);
-	return result;
-}
 
 static gboolean
 rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler)
@@ -1579,7 +1469,7 @@ rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler)
 			AudioscrobblerEntry *entry;
 			char *str;
 			entry = (AudioscrobblerEntry *) l->data;
-			str = rb_audioscrobbler_save_entry_to_string (entry);
+			str = rb_audioscrobbler_entry_save_to_string (entry);
 			result = gnome_vfs_write (handle, str, strlen (str), 
 						  NULL);
 			g_free (str);
@@ -1620,23 +1510,15 @@ rb_audioscrobbler_print_queue (RBAudioscrobbler *audioscrobbler, gboolean submis
 	}
 
 	for (; l != NULL; l = g_list_next (l)) {
-		char timestamp[30];
 		entry = (AudioscrobblerEntry *) l->data;
-		
-		rb_debug ("%-3d  artist: %s", ++i, entry->artist);
-		rb_debug ("      album: %s", entry->album);
-		rb_debug ("      title: %s", entry->title);
-		rb_debug ("     length: %d", entry->length);
-		strftime (timestamp, 30, SCROBBLER_DATE_FORMAT, 
-			  gmtime (&entry->play_time));
-		rb_debug ("  timestamp: %s", timestamp);
+		rb_audioscrobbler_entry_debug (entry, ++i);
 	}
 }
 
 static void
 rb_audioscrobbler_free_queue_entries (RBAudioscrobbler *audioscrobbler, GQueue **queue)
 {
-	g_queue_foreach (*queue, (GFunc) audioscrobbler_entry_free, NULL);
+	g_queue_foreach (*queue, (GFunc) rb_audioscrobbler_entry_free, NULL);
 	g_queue_free (*queue);
 	*queue = NULL;
 
