@@ -7,6 +7,14 @@
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
+ * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -318,6 +326,7 @@ impl_activate (RBPlugin *plugin,
 {
 	RBCdRecorderPlugin *pi = RB_CD_RECORDER_PLUGIN (plugin);
 	GtkUIManager       *uimanager = NULL;
+	GtkAction          *action;
 	int                 i;
 
 	pi->enabled = TRUE;
@@ -356,6 +365,13 @@ impl_activate (RBPlugin *plugin,
 				       FALSE);
 	}
 
+        action = gtk_action_group_get_action (pi->action_group,
+					      "MusicPlaylistBurnPlaylist");
+	/* Translators: this is the toolbar button label for */
+	/* Create Audio CD action                            */
+	g_object_set (action, "short-label", _("Burn"), NULL);
+
+
 	update_source (pi, shell);
 }
 
