@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -1294,6 +1302,9 @@ parse_meta (GHashTable *query)
 	const gchar *attrs;
 
 	attrs = g_hash_table_lookup (query, "meta");
+	if (attrs == NULL) {
+		return 0;
+	}
 	return parse_meta_str (attrs);
 }
 
@@ -1646,7 +1657,7 @@ databases_cb (RBDAAPShare *share,
 
 			mb.mlcl = rb_daap_structure_add (apso, RB_DAAP_CC_MLCL);
 
-			g_object_get (id->source, "query-model", &model, NULL);
+			g_object_get (id->source, "base-query-model", &model, NULL);
 			num_songs = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
 
 			rb_daap_structure_add (apso, RB_DAAP_CC_MTCO, (gint32) num_songs);
