@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,11 +38,6 @@ G_BEGIN_DECLS
 GType rb_daap_src_get_type (void);
 void rb_daap_src_set_plugin (RBPlugin *plugin);
 
-#ifdef HAVE_GSTREAMER_0_8
-void rb_daap_src_set_time (GstElement *element, glong time);
-glong rb_daap_src_get_time (GstElement *element);
-#endif
-
 G_END_DECLS
 
 #endif /* __RB_DAAP_SRC_H */
