@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -71,11 +79,16 @@ static void rb_ipod_db_queue_set_ipod_name (RbIpodDb *db,
 static void rb_ipod_db_queue_add_track (RbIpodDb *db, Itdb_Track *track);
 static void rb_ipod_db_queue_add_playlist (RbIpodDb *db, 
 					   Itdb_Playlist *playlist);
+static void rb_ipod_db_queue_remove_playlist (RbIpodDb *db, 
+					   Itdb_Playlist *playlist);
+static void rb_ipod_db_queue_rename_playlist (RbIpodDb *db, 
+					   Itdb_Playlist *playlist,
+					   const char *name);
 static void rb_ipod_db_queue_add_to_playlist (RbIpodDb *ipod_db, 
 					      Itdb_Playlist *playlist,
 					      Itdb_Track *track);
 static void rb_ipod_db_queue_remove_from_playlist (RbIpodDb *ipod_db, 
-						   Itdb_Playlist *playlist,
+						  Itdb_Playlist *playlist,
 						   Itdb_Track *track);
 static void rb_ipod_db_queue_set_thumbnail (RbIpodDb *db,
 					    Itdb_Track *track,
@@ -107,7 +120,8 @@ G_DEFINE_TYPE (RbIpodDb, rb_ipod_db, G_TYPE_OBJECT)
 
 #define IPOD_DB_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IPOD_DB, RbIpodDbPrivate))
 
-static void rb_itdb_save (RbIpodDb *ipod_db, GError **error)
+static void
+rb_itdb_save (RbIpodDb *ipod_db, GError **error)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -197,8 +211,10 @@ rb_ipod_db_class_init (RbIpodDbClass *klass)
 enum _RbIpodDelayedActionType {
 	RB_IPOD_ACTION_SET_NAME,
 	RB_IPOD_ACTION_ADD_TRACK,
-	RB_IPOD_ACTION_ADD_PLAYLIST,
 	RB_IPOD_ACTION_REMOVE_TRACK,
+	RB_IPOD_ACTION_ADD_PLAYLIST,
+	RB_IPOD_ACTION_REMOVE_PLAYLIST,
+	RB_IPOD_ACTION_RENAME_PLAYLIST,
 	RB_IPOD_ACTION_SET_THUMBNAIL,
 	RB_IPOD_ACTION_ADD_TO_PLAYLIST,
 	RB_IPOD_ACTION_REMOVE_FROM_PLAYLIST
@@ -228,7 +244,8 @@ struct _RbIpodDelayedAction {
 	};
 };
 
-static void rb_ipod_free_delayed_action (RbIpodDelayedAction *action) 
+static void
+rb_ipod_free_delayed_action (RbIpodDelayedAction *action) 
 {
 	switch (action->type) {
 	case RB_IPOD_ACTION_SET_NAME:
@@ -241,7 +258,13 @@ static void rb_ipod_free_delayed_action (RbIpodDelayedAction *action)
 		itdb_track_free (action->track);
 		break;
 	case RB_IPOD_ACTION_ADD_PLAYLIST:
-		itdb_playlist_free (action->playlist);
+		/* Do nothing */
+		break;
+	case RB_IPOD_ACTION_REMOVE_PLAYLIST:
+		/* Do nothing */
+		break;
+	case RB_IPOD_ACTION_RENAME_PLAYLIST:
+		g_free (action->name);
 		break;
 	case RB_IPOD_ACTION_REMOVE_TRACK:
 		/* Do nothing */
@@ -256,7 +279,8 @@ static void rb_ipod_free_delayed_action (RbIpodDelayedAction *action)
 	g_free (action);
 }
 
-const char *rb_ipod_db_get_ipod_name (RbIpodDb *db)
+const char *
+rb_ipod_db_get_ipod_name (RbIpodDb *db)
 {
 	Itdb_Playlist *mpl;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (db);
@@ -307,39 +331,14 @@ rb_ipod_db_remove_track_internal (RbIpodDb *ipod_db, Itdb_Track *track)
 }
 
 static void
-rb_ipod_db_set_thumbnail_internal (RbIpodDb *ipod_db, Itdb_Track *track, 
+rb_ipod_db_set_thumbnail_internal (RbIpodDb *ipod_db,
+				   Itdb_Track *track, 
 				   GdkPixbuf *pixbuf)
 {
-#ifndef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
-	gchar *image_data;
-	gsize image_data_len;
-	GError *err = NULL;
-	gboolean success;
-#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-
 	g_return_if_fail (track != NULL);
 	g_return_if_fail (pixbuf != NULL);
 
-#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
 	itdb_track_set_thumbnails_from_pixbuf (track, pixbuf);
-#else /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-
-	success = gdk_pixbuf_save_to_buffer (pixbuf,
-					     &image_data, &image_data_len,
-					     "jpeg", &err,
-					     "quality", "100",
-					     NULL);
-	if (!success) {
-		g_assert (image_data == NULL);
-		g_warning ("Failed to save pixbuf to buffer %s", err->message);
-		g_error_free (err);
-		return;
-	}
-
-	itdb_track_set_thumbnails_from_data (track, (guchar *) image_data, 
-					     image_data_len);
-	g_free (image_data);
-#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
 
 	rb_ipod_db_save_async (ipod_db);
 }
@@ -356,6 +355,24 @@ rb_ipod_db_add_playlist_internal (RbIpodDb *ipod_db,
 	rb_ipod_db_save_async (ipod_db);
 }
 
+static void
+rb_ipod_db_remove_playlist_internal (RbIpodDb *ipod_db, 
+				     Itdb_Playlist *playlist)
+{
+	itdb_playlist_remove (playlist);
+	rb_ipod_db_save_async (ipod_db);
+}
+
+static void
+rb_ipod_db_rename_playlist_internal (RbIpodDb *ipod_db, 
+				     Itdb_Playlist *playlist,
+				     const char *name)
+{
+	g_free (playlist->name);
+	playlist->name = g_strdup (name);
+	rb_ipod_db_save_async (ipod_db);
+}
+
 static void
 rb_ipod_db_add_track_internal (RbIpodDb *ipod_db, Itdb_Track *track)
 {
@@ -387,8 +404,10 @@ rb_ipod_db_remove_from_playlist_internal (RbIpodDb* ipod_db,
 }
 
 
-void rb_ipod_db_set_thumbnail (RbIpodDb* ipod_db, Itdb_Track *track, 
-			       GdkPixbuf *pixbuf)
+void
+rb_ipod_db_set_thumbnail (RbIpodDb* ipod_db,
+			  Itdb_Track *track, 
+			  GdkPixbuf *pixbuf)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -399,7 +418,8 @@ void rb_ipod_db_set_thumbnail (RbIpodDb* ipod_db, Itdb_Track *track,
 	}
 }
 
-void rb_ipod_db_add_track (RbIpodDb* ipod_db, Itdb_Track *track)
+void
+rb_ipod_db_add_track (RbIpodDb* ipod_db, Itdb_Track *track)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -410,7 +430,8 @@ void rb_ipod_db_add_track (RbIpodDb* ipod_db, Itdb_Track *track)
 	}
 }
 
-void rb_ipod_db_add_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist)
+void
+rb_ipod_db_add_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -421,7 +442,33 @@ void rb_ipod_db_add_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist)
 	}
 }
 
-void rb_ipod_db_remove_track (RbIpodDb* ipod_db, Itdb_Track *track)
+void
+rb_ipod_db_remove_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist)
+{
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+
+	if (priv->read_only) {
+		rb_ipod_db_queue_remove_playlist (ipod_db, playlist);
+	} else {
+		rb_ipod_db_remove_playlist_internal (ipod_db, playlist);
+	}
+}
+
+void
+rb_ipod_db_rename_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist, const char *name)
+{
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+
+	if (priv->read_only) {
+		rb_ipod_db_queue_rename_playlist (ipod_db, playlist, name);
+	} else {
+		rb_ipod_db_rename_playlist_internal (ipod_db, playlist, name);
+	}
+}
+
+
+void
+rb_ipod_db_remove_track (RbIpodDb* ipod_db, Itdb_Track *track)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -432,7 +479,8 @@ void rb_ipod_db_remove_track (RbIpodDb* ipod_db, Itdb_Track *track)
 	}
 }
 
-void rb_ipod_db_set_ipod_name (RbIpodDb *ipod_db, const char *name)
+void
+rb_ipod_db_set_ipod_name (RbIpodDb *ipod_db, const char *name)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -443,7 +491,8 @@ void rb_ipod_db_set_ipod_name (RbIpodDb *ipod_db, const char *name)
 	}
 }
 
-void rb_ipod_db_add_to_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist,
+void
+rb_ipod_db_add_to_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist,
 				 Itdb_Track *track)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -455,9 +504,10 @@ void rb_ipod_db_add_to_playlist (RbIpodDb* ipod_db, Itdb_Playlist *playlist,
 	}	
 }
 
-void rb_ipod_db_remove_from_playlist (RbIpodDb* ipod_db, 
-				      Itdb_Playlist *playlist,
-				      Itdb_Track *track)
+void
+rb_ipod_db_remove_from_playlist (RbIpodDb* ipod_db, 
+				 Itdb_Playlist *playlist,
+				 Itdb_Track *track)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -517,11 +567,17 @@ rb_ipod_db_process_delayed_actions (RbIpodDb *ipod_db)
 			rb_debug ("IPOD_ACTION_ADD_PLAYLIST");
 			rb_ipod_db_add_playlist_internal (ipod_db, 
 							  action->playlist);
-			/* The playlist was added to the iPod database, 
-			 * 'action' is no longer responsible for its memory 
-			 * handling
-			 */
-			action->playlist = NULL;
+			break;
+		case RB_IPOD_ACTION_REMOVE_PLAYLIST:
+			rb_debug ("IPOD_ACTION_REMOVE_PLAYLIST");
+			rb_ipod_db_remove_playlist_internal (ipod_db, 
+							     action->playlist);
+			break;
+		case RB_IPOD_ACTION_RENAME_PLAYLIST:
+			rb_debug ("IPOD_ACTION_RENAME_PLAYLIST");
+			rb_ipod_db_rename_playlist_internal (ipod_db, 
+							     action->playlist,
+							     action->name);
 			break;
 		case RB_IPOD_ACTION_ADD_TO_PLAYLIST:
 			rb_debug ("IPOD_ACTION_ADD_TO_PLAYLIST");
@@ -541,8 +597,9 @@ rb_ipod_db_process_delayed_actions (RbIpodDb *ipod_db)
 	}
 }
 
-static void rb_ipod_db_queue_remove_track (RbIpodDb *ipod_db,
-					   Itdb_Track *track)
+static void
+rb_ipod_db_queue_remove_track (RbIpodDb *ipod_db,
+			       Itdb_Track *track)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -555,8 +612,9 @@ static void rb_ipod_db_queue_remove_track (RbIpodDb *ipod_db,
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_set_ipod_name (RbIpodDb *ipod_db, 
-					    const char *new_name)
+static void
+rb_ipod_db_queue_set_ipod_name (RbIpodDb *ipod_db, 
+				const char *new_name)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -569,21 +627,55 @@ static void rb_ipod_db_queue_set_ipod_name (RbIpodDb *ipod_db,
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_add_playlist (RbIpodDb *ipod_db,
-					   Itdb_Playlist *playlist)
+static void
+rb_ipod_db_queue_add_playlist (RbIpodDb *ipod_db,
+			       Itdb_Playlist *playlist)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 	
 	g_assert (priv->read_only);
-	rb_debug ("Queueing add track action since the iPod database is currently read-only");
+	rb_debug ("Queueing add playlist action since the iPod database is currently read-only");
 	action = g_new0 (RbIpodDelayedAction, 1);
 	action->type = RB_IPOD_ACTION_ADD_PLAYLIST;
 	action->playlist = playlist;
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_add_track (RbIpodDb *ipod_db, Itdb_Track *track)
+static void
+rb_ipod_db_queue_remove_playlist (RbIpodDb *ipod_db,
+				  Itdb_Playlist *playlist)
+{
+	RbIpodDelayedAction *action;
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+	
+	g_assert (priv->read_only);
+	rb_debug ("Queueing remove playlist action since the iPod database is currently read-only");
+	action = g_new0 (RbIpodDelayedAction, 1);
+	action->type = RB_IPOD_ACTION_REMOVE_PLAYLIST;
+	action->playlist = playlist;
+	g_queue_push_tail (priv->delayed_actions, action);
+}
+
+static void
+rb_ipod_db_queue_rename_playlist (RbIpodDb *ipod_db,
+				  Itdb_Playlist *playlist,
+				  const char *name)
+{
+	RbIpodDelayedAction *action;
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+	
+	g_assert (priv->read_only);
+	rb_debug ("Queueing rename playlist action since the iPod database is currently read-only");
+	action = g_new0 (RbIpodDelayedAction, 1);
+	action->type = RB_IPOD_ACTION_RENAME_PLAYLIST;
+	action->playlist = playlist;
+	action->name = g_strdup (name);
+	g_queue_push_tail (priv->delayed_actions, action);
+}
+
+static void
+rb_ipod_db_queue_add_track (RbIpodDb *ipod_db, Itdb_Track *track)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -596,9 +688,10 @@ static void rb_ipod_db_queue_add_track (RbIpodDb *ipod_db, Itdb_Track *track)
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_add_to_playlist (RbIpodDb *ipod_db, 
-					      Itdb_Playlist *playlist,
-					      Itdb_Track *track)
+static void
+rb_ipod_db_queue_add_to_playlist (RbIpodDb *ipod_db, 
+				  Itdb_Playlist *playlist,
+				  Itdb_Track *track)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -612,9 +705,10 @@ static void rb_ipod_db_queue_add_to_playlist (RbIpodDb *ipod_db,
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_remove_from_playlist (RbIpodDb *ipod_db, 
-						   Itdb_Playlist *playlist,
-						   Itdb_Track *track)
+static void
+rb_ipod_db_queue_remove_from_playlist (RbIpodDb *ipod_db, 
+				       Itdb_Playlist *playlist,
+				       Itdb_Track *track)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -628,9 +722,10 @@ static void rb_ipod_db_queue_remove_from_playlist (RbIpodDb *ipod_db,
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static void rb_ipod_db_queue_set_thumbnail (RbIpodDb *ipod_db,
-					    Itdb_Track *track,
-					    GdkPixbuf *pixbuf)
+static void
+rb_ipod_db_queue_set_thumbnail (RbIpodDb *ipod_db,
+				Itdb_Track *track,
+				GdkPixbuf *pixbuf)
 {
 	RbIpodDelayedAction *action;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
@@ -658,7 +753,8 @@ rb_ipod_db_get_volume_path (GnomeVFSVolume *volume)
 	return path;
 }
 
-static gboolean rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
+static gboolean
+rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
 {
 	char *mount_path;
 	const Itdb_IpodInfo *info;
@@ -674,7 +770,9 @@ static gboolean rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
 
 	info = itdb_device_get_ipod_info(priv->itdb->device);
 	if (info->ipod_generation == ITDB_IPOD_GENERATION_UNKNOWN ||
-	    info->ipod_model == ITDB_IPOD_MODEL_SHUFFLE) {
+	    info->ipod_generation == ITDB_IPOD_GENERATION_SHUFFLE_1 ||
+	    info->ipod_generation == ITDB_IPOD_GENERATION_SHUFFLE_2 ||
+	    info->ipod_generation == ITDB_IPOD_GENERATION_SHUFFLE_3) {
 		priv->needs_shuffle_db = TRUE;
 	} else {
 		priv->needs_shuffle_db = FALSE;
@@ -683,7 +781,8 @@ static gboolean rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
         return TRUE;
 }
 
-RbIpodDb *rb_ipod_db_new (GnomeVFSVolume *volume)
+RbIpodDb *
+rb_ipod_db_new (GnomeVFSVolume *volume)
 {
 	RbIpodDb *db;
         gboolean success;
@@ -725,7 +824,8 @@ ipod_db_saved_idle_cb (RbIpodDb *ipod_db)
 	return FALSE;
 }
 
-static gpointer saving_thread (RbIpodDb *ipod_db)
+static gpointer
+saving_thread (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
@@ -773,28 +873,32 @@ rb_ipod_db_save_async (RbIpodDb *ipod_db)
 	}
 }
 
-GList *rb_ipod_db_get_playlists (RbIpodDb *ipod_db)
+GList *
+rb_ipod_db_get_playlists (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
 	return priv->itdb->playlists;
 }
 
-GList *rb_ipod_db_get_tracks (RbIpodDb *ipod_db)
+GList *
+rb_ipod_db_get_tracks (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
 	return priv->itdb->tracks;
 }
 
-const char *rb_ipod_db_get_mount_path (RbIpodDb *ipod_db)
+const char *
+rb_ipod_db_get_mount_path (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
 	return itdb_get_mountpoint (priv->itdb);
 }
 
-Itdb_Device *rb_ipod_db_get_device (RbIpodDb *ipod_db)
+Itdb_Device *
+rb_ipod_db_get_device (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 	if (priv->itdb == NULL) {
@@ -803,3 +907,4 @@ Itdb_Device *rb_ipod_db_get_device (RbIpodDb *ipod_db)
 
 	return priv->itdb->device;
 }
+
