@@ -8,6 +8,14 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
+# The Rhythmbox authors hereby grants permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and Rhythmbox. This permission is above and beyond the permissions granted
+# by the GPL license by which Rhythmbox is covered. If you modify this code
+# you may extend this exception to your version of the code, but you are not
+# obligated to do so. If you do not wish to do so, delete this exception
+# statement from your version.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,7 +44,7 @@ has_gnome_keyring = False
 #	pass
 
 
-magnatune_partner_id = "zimmerman"
+magnatune_partner_id = "rhythmbox"
 
 # URIs
 magnatune_dir = gnome.user_dir_get() + "rhythmbox/magnatune/"
@@ -81,16 +89,10 @@ class MagnatuneSource(rb.BrowserSource):
 		self.__download_progress = 0.0 # progress of current download(s)
 		self.purchase_filesize = 0 # total amount of bytes to download
 
+
 	def do_set_property(self, property, value):
 		if property.name == 'plugin':
 			self.__plugin = value
-
-			# we have to wait until we get the plugin to do this
-			circle_file_name = self.__plugin.find_file("magnatune_circle_small.png")
-			width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)
-			icon = gtk.gdk.pixbuf_new_from_file_at_size(circle_file_name, width, height)
-			self.set_property("icon", icon)
-
 		else:
 			raise AttributeError, 'unknown property %s' % property.name
 
@@ -336,6 +338,12 @@ class MagnatuneSource(rb.BrowserSource):
 		self.__notify_status_changed()
 		self.__load_handle = gnomevfs.async.open (local_song_info_uri, self.__load_catalogue_open_cb)
 
+	def __find_song_info(self, catalogue):
+		for info in catalogue.infolist():
+			if info.filename.endswith("song_info.xml"):
+				return info.filename;
+		return None
+
 	def __download_update_cb (self, _reserved, info, moving):
 		self.__load_current_size = info.bytes_copied
 		self.__load_total_size = info.bytes_total
@@ -345,7 +353,11 @@ class MagnatuneSource(rb.BrowserSource):
 			# done downloading, unzip to real location
 			catalog = zipfile.ZipFile(local_song_info_temp_uri.path)
 			out = create_if_needed(local_song_info_uri, gnomevfs.OPEN_WRITE)
-			out.write(catalog.read("opt/magnatune/info/song_info.xml"))
+			filename = self.__find_song_info(catalog)
+			if filename is None:
+				rb.error_dialog(title="Unable to load catalogue", message=_("Rhythmbox could not understand the Magnatune catalogue, please file a bug."))
+				return
+			out.write(catalog.read(filename))
 			out.close()
 			catalog.close()
 			gnomevfs.unlink(local_song_info_temp_uri)
