@@ -8,6 +8,14 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -117,7 +125,7 @@ static void     plugin_name##_class_intern_init (gpointer klass)		\
 G_MODULE_EXPORT GType								\
 register_rb_plugin (GTypeModule *module)					\
 {										\
-	static const GTypeInfo our_info =					\
+	const GTypeInfo our_info =						\
 	{									\
 		sizeof (PluginName##Class),					\
 		NULL, /* base_init */						\
@@ -175,7 +183,7 @@ type_name##_register_type (GTypeModule *module) 				\
 { 										\
 										\
 	if ((type_name##_type_id == 0)) { 					\
-		static const GTypeInfo g_define_type_info = { 			\
+		const GTypeInfo g_define_type_info = { 				\
 			sizeof (TypeName##Class), 				\
 			(GBaseInitFunc) ((void *)0), 				\
 			(GBaseFinalizeFunc) ((void *)0), 			\
