@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Headfile of podcast properties dialog
  *
  *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org>
@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,7 +28,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <time.h>
@@ -40,14 +48,15 @@
 
 static void rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass);
 static void rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog);
+static void rb_podcast_properties_dialog_dispose (GObject *object);
 static void rb_podcast_properties_dialog_finalize (GObject *object);
-static void rb_podcast_properties_dialog_set_property (GObject *object, 
+static void rb_podcast_properties_dialog_set_property (GObject *object,
 						       guint prop_id,
-						       const GValue *value, 
+						       const GValue *value,
 						       GParamSpec *pspec);
-static void rb_podcast_properties_dialog_get_property (GObject *object, 
+static void rb_podcast_properties_dialog_get_property (GObject *object,
 						       guint prop_id,
-						       GValue *value, 
+						       GValue *value,
 						       GParamSpec *pspec);
 static gboolean rb_podcast_properties_dialog_get_current_entry (RBPodcastPropertiesDialog *dialog);
 static void rb_podcast_properties_dialog_response_cb (GtkDialog *gtkdialog,
@@ -89,11 +98,11 @@ struct RBPodcastPropertiesDialogPrivate
 	GtkWidget   *rating;
 	GtkWidget   *date;
 	GtkWidget   *description;
-	
+
 	GtkWidget   *close_button;
 };
 
-enum 
+enum
 {
 	PROP_0,
 	PROP_ENTRY_VIEW,
@@ -102,7 +111,6 @@ enum
 
 G_DEFINE_TYPE (RBPodcastPropertiesDialog, rb_podcast_properties_dialog, GTK_TYPE_DIALOG)
 
-
 static void
 rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 {
@@ -119,6 +127,7 @@ rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 					                      RB_TYPE_ENTRY_VIEW,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	object_class->dispose = rb_podcast_properties_dialog_dispose;
 	object_class->finalize = rb_podcast_properties_dialog_finalize;
 
 	g_type_class_add_private (klass, sizeof (RBPodcastPropertiesDialogPrivate));
@@ -128,9 +137,9 @@ static void
 rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 {
 	GladeXML *xml;
-	
-	dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog, 
-						    RB_TYPE_PODCAST_PROPERTIES_DIALOG, 
+
+	dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
+						    RB_TYPE_PODCAST_PROPERTIES_DIALOG,
 						    RBPodcastPropertiesDialogPrivate);
 
 	g_signal_connect_object (G_OBJECT (dialog),
@@ -182,7 +191,7 @@ rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 	rb_glade_boldify_label (xml, "descriptionDescLabel");
 
 	dialog->priv->rating = GTK_WIDGET (rb_rating_new ());
-	g_signal_connect_object (dialog->priv->rating, 
+	g_signal_connect_object (dialog->priv->rating,
 				 "rated",
 				 G_CALLBACK (rb_podcast_properties_dialog_rated_cb),
 				 G_OBJECT (dialog), 0);
@@ -191,6 +200,26 @@ rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 	g_object_unref (G_OBJECT (xml));
 }
 
+static void
+rb_podcast_properties_dialog_dispose (GObject *object)
+{
+	RBPodcastPropertiesDialog *dialog;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_PODCAST_PROPERTIES_DIALOG (object));
+
+	dialog = RB_PODCAST_PROPERTIES_DIALOG (object);
+
+	g_return_if_fail (dialog->priv != NULL);
+
+	if (dialog->priv->db != NULL) {
+		g_object_unref (dialog->priv->db);
+		dialog->priv->db = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_podcast_properties_dialog_parent_class)->dispose (object);
+}
+
 static void
 rb_podcast_properties_dialog_finalize (GObject *object)
 {
@@ -206,6 +235,22 @@ rb_podcast_properties_dialog_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_podcast_properties_dialog_parent_class)->finalize (object);
 }
 
+static void
+rb_podcast_properties_dialog_set_entry_view (RBPodcastPropertiesDialog *dialog,
+					     RBEntryView               *view)
+{
+	if (dialog->priv->db != NULL) {
+		g_object_unref (dialog->priv->db);
+	}
+
+	dialog->priv->entry_view = view;
+
+	if (dialog->priv->entry_view != NULL) {
+		g_object_get (dialog->priv->entry_view,
+			      "db", &dialog->priv->db, NULL);
+	}
+}
+
 static void
 rb_podcast_properties_dialog_set_property (GObject *object,
 					   guint prop_id,
@@ -216,9 +261,7 @@ rb_podcast_properties_dialog_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_VIEW:
-		dialog->priv->entry_view = g_value_get_object (value);
-		g_object_get (G_OBJECT (dialog->priv->entry_view), "db",
-			      &dialog->priv->db, NULL);
+		rb_podcast_properties_dialog_set_entry_view (dialog, g_value_get_object (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -278,7 +321,7 @@ static gboolean
 rb_podcast_properties_dialog_get_current_entry (RBPodcastPropertiesDialog *dialog)
 {
 	GList *selected_entries;
-	
+
 	/* get the entry */
 	selected_entries = rb_entry_view_get_selected_entries (dialog->priv->entry_view);
 
@@ -315,7 +358,7 @@ rb_podcast_properties_dialog_update_title (RBPodcastPropertiesDialog *dialog)
 {
 	const char *name;
 	char *tmp;
-	
+
 	name = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_TITLE);
 	tmp = g_strdup_printf (_("%s Properties"), name);
 	gtk_window_set_title (GTK_WINDOW (dialog), tmp);
@@ -345,7 +388,7 @@ rb_podcast_properties_dialog_update_duration (RBPodcastPropertiesDialog *dialog)
 {
         char *text;
         gulong duration = 0;
-	
+
         duration = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_DURATION);
 
 	text = rb_make_duration_string (duration);
@@ -458,7 +501,7 @@ rb_podcast_properties_dialog_update_date (RBPodcastPropertiesDialog *dialog)
 {
 	gulong post_time;
 	char *time;
-	
+
 	post_time = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_POST_TIME);
 	time = rb_podcast_properties_dialog_parse_time (post_time);
 
