@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -192,7 +200,7 @@ rb_refstring_get_sort_key (RBRefString *val)
 		const char *s;
 
 		s = rb_refstring_get_folded (val);
-		newstring = rb_utf8_collate_key_for_filename (s, -1);
+		newstring = g_utf8_collate_key_for_filename (s, -1);
 
 		if (g_atomic_pointer_compare_and_exchange (ptr, NULL, newstring)) {
 			string = newstring;
@@ -218,3 +226,17 @@ rb_refstring_equal (gconstpointer ap, gconstpointer bp)
 {
 	return (ap == bp);
 }
+
+GType
+rb_refstring_get_type (void)
+{
+	static GType type = 0;
+
+	if (G_UNLIKELY (type == 0)) {
+		type = g_boxed_type_register_static ("RBRefString",
+						     (GBoxedCopyFunc)rb_refstring_ref,
+						     (GBoxedFreeFunc)rb_refstring_unref);
+	}
+
+	return type;
+}
