@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -86,7 +94,7 @@ struct RhythmDBEntry_ {
 	RBRefString *musicbrainz_artistid;
 	RBRefString *musicbrainz_albumid;
 	RBRefString *musicbrainz_albumartistid;
-	RBRefString *musicbrainz_artistsortname;
+	RBRefString *artist_sortname;
 	gulong tracknum;
 	gulong discnum;
 	gulong duration;
@@ -120,7 +128,7 @@ struct RhythmDBEntry_ {
 	RBRefString *playback_error;
 };
 
-struct RhythmDBPrivate
+struct _RhythmDBPrivate
 {
 	char *name;
 
