@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -78,6 +86,8 @@ typedef struct
 					 RhythmDBEntry *entry);
 	void	(*post_entry_delete)	(RhythmDBQueryModel *model,
 					 RhythmDBEntry *entry);
+	gboolean (*filter_entry_drop)	(RhythmDBQueryModel *model,
+					 RhythmDBEntry *entry);
 
 } RhythmDBQueryModelClass;
 
@@ -106,6 +116,8 @@ void			rhythmdb_query_model_add_entry		(RhythmDBQueryModel *model,
 gboolean		rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model,
 								 RhythmDBEntry *entry);
 
+void			rhythmdb_query_model_shuffle_entries 	(RhythmDBQueryModel *model);
+
 void			rhythmdb_query_model_move_entry 	(RhythmDBQueryModel *model,
 								 RhythmDBEntry *entry,
 								 gint index);
