@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,12 +37,13 @@
 
 #include "config.h"
 #include "rb-refstring.h"
+#include "rb-string-value-map.h"
 #include "rhythmdb-query-results.h"
 
 G_BEGIN_DECLS
 
-struct RhythmDB_;
-typedef struct RhythmDB_ RhythmDB;
+typedef struct _RhythmDB RhythmDB;
+typedef struct _RhythmDBClass RhythmDBClass;
 
 #define RHYTHMDB_TYPE      (rhythmdb_get_type ())
 #define RHYTHMDB(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE, RhythmDB))
@@ -55,19 +64,22 @@ typedef char* (*RhythmDBEntryStringFunc) (RhythmDBEntry *entry, gpointer data);
 typedef gboolean (*RhythmDBEntryCanSyncFunc) (RhythmDB *db, RhythmDBEntry *entry, gpointer data);
 typedef void (*RhythmDBEntrySyncFunc) (RhythmDB *db, RhythmDBEntry *entry, GError **error, gpointer data);
 
-enum RhythmDBEntryCategory {
+GType rhythmdb_entry_category_get_type (void);
+#define RHYTHMDB_TYPE_ENTRY_CATEGORY (rhythmdb_entry_category_get_type ())
+typedef enum {
 	RHYTHMDB_ENTRY_NORMAL,		/* anything that doesn't match the other categories */
 	RHYTHMDB_ENTRY_STREAM,		/* endless streams (eg shoutcast, last.fm) */
 	RHYTHMDB_ENTRY_CONTAINER,	/* things that point to other entries (eg podcast feeds) */
 	RHYTHMDB_ENTRY_VIRTUAL		/* import errors, ignored files */
-};
+} RhythmDBEntryCategory;
 
 typedef struct {
 	char 				*name;
 
 	guint				entry_type_data_size;
 	gboolean			save_to_disk;
-	enum RhythmDBEntryCategory	category;
+	gboolean			has_playlists;
+	RhythmDBEntryCategory		category;
 
 	/* virtual functions here */
 	RhythmDBEntryActionFunc		post_entry_create;
@@ -182,6 +194,7 @@ typedef enum
 	/* synthetic properties */
 	RHYTHMDB_PROP_SEARCH_MATCH,
 	RHYTHMDB_PROP_YEAR,
+	RHYTHMDB_PROP_KEYWORD, /**/
 
 	/* Podcast properties */
 	RHYTHMDB_PROP_STATUS,
@@ -194,6 +207,10 @@ typedef enum
 	RHYTHMDB_PROP_POST_TIME,
 
 	RHYTHMDB_PROP_MUSICBRAINZ_TRACKID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID,
+	RHYTHMDB_PROP_ARTIST_SORTNAME,
 
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
@@ -209,6 +226,8 @@ enum {
 #define RHYTHMDB_PROP_STREAM_SONG_TITLE		"rb:stream-song-title"
 #define RHYTHMDB_PROP_STREAM_SONG_ARTIST	"rb:stream-song-artist"
 #define RHYTHMDB_PROP_STREAM_SONG_ALBUM		"rb:stream-song-album"
+#define RHYTHMDB_PROP_COVER_ART			"rb:coverArt"
+#define RHYTHMDB_PROP_COVER_ART_URI		"rb:coverArt-uri"
 
 GType rhythmdb_query_type_get_type (void);
 GType rhythmdb_prop_type_get_type (void);
@@ -220,7 +239,7 @@ typedef struct {
 	guint type;
 	guint propid;
 	GValue *val;
-	GPtrArray *subquery;
+	RhythmDBQuery *subquery;
 } RhythmDBQueryData;
 
 typedef struct {
@@ -249,16 +268,16 @@ typedef enum
 
 GQuark rhythmdb_error_quark (void);
 
-typedef struct RhythmDBPrivate RhythmDBPrivate;
+typedef struct _RhythmDBPrivate RhythmDBPrivate;
 
-struct RhythmDB_
+struct _RhythmDB
 {
 	GObject parent;
 
 	RhythmDBPrivate *priv;
 };
 
-typedef struct
+struct _RhythmDBClass
 {
 	GObjectClass parent;
 
@@ -266,8 +285,10 @@ typedef struct
 	void	(*entry_added)		(RhythmDB *db, RhythmDBEntry *entry);
 	void	(*entry_changed)	(RhythmDB *db, RhythmDBEntry *entry, GSList *changes); /* list of RhythmDBEntryChanges */
 	void	(*entry_deleted)	(RhythmDB *db, RhythmDBEntry *entry);
+	void	(*entry_keyword_added)	(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
+	void	(*entry_keyword_removed)(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
 	GValue *(*entry_extra_metadata_request) (RhythmDB *db, RhythmDBEntry *entry);
-	void    (*entry_extra_metadata_gather) (RhythmDB *db, RhythmDBEntry *entry, GHashTable *data);
+	void    (*entry_extra_metadata_gather) (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data);
 	void	(*entry_extra_metadata_notify) (RhythmDB *db, RhythmDBEntry *entry, const char *field, GValue *metadata);
 	void	(*load_complete)	(RhythmDB *db);
 	void	(*save_complete)	(RhythmDB *db);
@@ -296,7 +317,7 @@ typedef struct
 
 	RhythmDBEntry *	(*impl_lookup_by_id)    (RhythmDB *db, gint id);
 
-	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, GPtrArray *query, RhythmDBEntry *entry);
+	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, RhythmDBQuery *query, RhythmDBEntry *entry);
 
 	void		(*impl_entry_foreach)	(RhythmDB *db, GFunc func, gpointer data);
 
@@ -306,14 +327,26 @@ typedef struct
 
 	gint64		(*impl_entry_count_by_type) (RhythmDB *db, RhythmDBEntryType type);
 
-	void		(*impl_do_full_query)	(RhythmDB *db, GPtrArray *query,
+	void		(*impl_do_full_query)	(RhythmDB *db, RhythmDBQuery *query,
 						 RhythmDBQueryResults *results,
 						 gboolean *cancel);
 
 	void		(*impl_entry_type_registered) (RhythmDB *db,
 						       const char *name,
 						       RhythmDBEntryType type);
-} RhythmDBClass;
+
+	gboolean	(*impl_entry_keyword_add)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	gboolean	(*impl_entry_keyword_remove)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	gboolean	(*impl_entry_keyword_has)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	GList*		(*impl_entry_keywords_get)	(RhythmDB *db,
+							 RhythmDBEntry *entry);
+};
 
 GType		rhythmdb_get_type	(void);
 
@@ -336,7 +369,11 @@ RhythmDBEntry *	rhythmdb_entry_new	(RhythmDB *db, RhythmDBEntryType type, const
 RhythmDBEntry *	rhythmdb_entry_example_new	(RhythmDB *db, RhythmDBEntryType type, const char *uri);
 
 void		rhythmdb_add_uri	(RhythmDB *db, const char *uri);
-void		rhythmdb_add_uri_with_type	(RhythmDB *db, const char *uri, RhythmDBEntryType type);
+void		rhythmdb_add_uri_with_types (RhythmDB *db,
+					     const char *uri,
+					     RhythmDBEntryType type,
+					     RhythmDBEntryType ignore_type,
+					     RhythmDBEntryType error_type);
 
 void		rhythmdb_entry_get	(RhythmDB *db, RhythmDBEntry *entry, RhythmDBPropType propid, GValue *val);
 void		rhythmdb_entry_set	(RhythmDB *db, RhythmDBEntry *entry,
@@ -359,7 +396,7 @@ RhythmDBEntry *	rhythmdb_entry_lookup_by_id     (RhythmDB *db, gint id);
 
 RhythmDBEntry * rhythmdb_entry_lookup_from_string (RhythmDB *db, const char *str, gboolean is_id);
 
-gboolean	rhythmdb_evaluate_query		(RhythmDB *db, GPtrArray *query,
+gboolean	rhythmdb_evaluate_query		(RhythmDB *db, RhythmDBQuery *query,
 						 RhythmDBEntry *entry);
 
 void		rhythmdb_entry_foreach		(RhythmDB *db,
@@ -374,7 +411,19 @@ void		rhythmdb_entry_foreach_by_type  (RhythmDB *db,
 gint64		rhythmdb_entry_count_by_type	(RhythmDB *db,
 						 RhythmDBEntryType entry_type);
 
-/**
+gboolean	rhythmdb_entry_keyword_add	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+gboolean	rhythmdb_entry_keyword_remove	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+gboolean	rhythmdb_entry_keyword_has	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+GList* /*<RBRefString>*/ rhythmdb_entry_keywords_get	(RhythmDB *db,
+							 RhythmDBEntry *entry);
+
+/*
  * Returns a freshly allocated GtkTreeModel which represents the query.
  * The extended arguments alternate between RhythmDBQueryType args
  * and their values. Items are prioritized like algebraic expressions, and
@@ -396,7 +445,7 @@ void		rhythmdb_do_full_query			(RhythmDB *db,
 							 ...);
 void		rhythmdb_do_full_query_parsed		(RhythmDB *db,
 							 RhythmDBQueryResults *results,
-							 GPtrArray *query);
+							 RhythmDBQuery *query);
 
 void		rhythmdb_do_full_query_async		(RhythmDB *db,
 							 RhythmDBQueryResults *results,
@@ -404,24 +453,25 @@ void		rhythmdb_do_full_query_async		(RhythmDB *db,
 
 void		rhythmdb_do_full_query_async_parsed	(RhythmDB *db,
 							 RhythmDBQueryResults *results,
-							 GPtrArray *query);
+							 RhythmDBQuery *query);
 
-GPtrArray *	rhythmdb_query_parse			(RhythmDB *db, ...);
-void		rhythmdb_query_append			(RhythmDB *db, GPtrArray *query, ...);
-void		rhythmdb_query_append_prop_multiple	(RhythmDB *db, GPtrArray *query, RhythmDBPropType propid, GList *items);
-void		rhythmdb_query_concatenate		(GPtrArray *query1, GPtrArray *query2);
-void		rhythmdb_query_free			(GPtrArray *query);
-GPtrArray *	rhythmdb_query_copy			(GPtrArray *array);
-void		rhythmdb_query_preprocess		(RhythmDB *db, GPtrArray *query);
+RhythmDBQuery *	rhythmdb_query_parse			(RhythmDB *db, ...);
+void		rhythmdb_query_append			(RhythmDB *db, RhythmDBQuery *query, ...);
+void		rhythmdb_query_append_params		(RhythmDB *db, RhythmDBQuery *query, RhythmDBQueryType type, RhythmDBPropType prop, const GValue *value);
+void		rhythmdb_query_append_prop_multiple	(RhythmDB *db, RhythmDBQuery *query, RhythmDBPropType propid, GList *items);
+void		rhythmdb_query_concatenate		(RhythmDBQuery *query1, RhythmDBQuery *query2);
+void		rhythmdb_query_free			(RhythmDBQuery *query);
+RhythmDBQuery *	rhythmdb_query_copy			(RhythmDBQuery *array);
+void		rhythmdb_query_preprocess		(RhythmDB *db, RhythmDBQuery *query);
 
-void		rhythmdb_query_serialize		(RhythmDB *db, GPtrArray *query,
+void		rhythmdb_query_serialize		(RhythmDB *db, RhythmDBQuery *query,
 							 xmlNodePtr node);
 
-GPtrArray *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
+RhythmDBQuery *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
 
-char *		rhythmdb_query_to_string		(RhythmDB *db, GPtrArray *query);
+char *		rhythmdb_query_to_string		(RhythmDB *db, RhythmDBQuery *query);
 
-gboolean	rhythmdb_query_is_time_relative		(RhythmDB *db, GPtrArray *query);
+gboolean	rhythmdb_query_is_time_relative		(RhythmDB *db, RhythmDBQuery *query);
 
 const xmlChar *	rhythmdb_nice_elt_name_from_propid	(RhythmDB *db, RhythmDBPropType propid);
 int		rhythmdb_propid_from_nice_elt_name	(RhythmDB *db, const xmlChar *name);
@@ -430,7 +480,7 @@ void		rhythmdb_emit_entry_added		(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_deleted		(RhythmDB *db, RhythmDBEntry *entry);
 
 GValue *	rhythmdb_entry_request_extra_metadata	(RhythmDB *db, RhythmDBEntry *entry, const gchar *property_name);
-GHashTable *	rhythmdb_entry_gather_metadata		(RhythmDB *db, RhythmDBEntry *entry);
+RBStringValueMap* rhythmdb_entry_gather_metadata	(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_extra_metadata_notify (RhythmDB *db, RhythmDBEntry *entry, const gchar *property_name, const GValue *metadata);
 
 gboolean	rhythmdb_is_busy			(RhythmDB *db);
