@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -23,6 +31,17 @@
 
 #include <config.h>
 
+#ifdef ENABLE_PYTHON
+/* pyconfig.h usually defines _XOPEN_SOURCE */
+#undef _XOPEN_SOURCE
+#include <pygobject.h>
+
+/* make sure it's defined somehow */
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE
+#endif
+#endif
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
@@ -76,6 +95,8 @@ static char **remaining_args    = NULL;
 
 static gboolean load_uri_args (const char **args, GFunc handler, gpointer user_data);
 static void dbus_load_uri (const char *filename, DBusGProxy *proxy);
+static void removable_media_scan_finished (RBShell *shell, gpointer data);
+static void local_load_uri (const char *filename, RBShell *shell);
 
 static void main_shell_weak_ref_cb (gpointer data, GObject *objptr);
 
@@ -271,6 +292,11 @@ main (int argc, char **argv)
 			obj = rb_shell_get_playlist_manager (rb_shell);
 			path = rb_shell_get_playlist_manager_path (rb_shell);
 			dbus_g_connection_register_g_object (session_bus, path, obj);
+
+			g_signal_connect (G_OBJECT (rb_shell),
+					  "removable_media_scan_finished",
+					  G_CALLBACK (removable_media_scan_finished),
+					  NULL);
 		}
 	} else if (!no_registration && session_bus != NULL) {
 		DBusGProxy *shell_proxy;
@@ -303,7 +329,13 @@ main (int argc, char **argv)
 	} else {
 
 		rb_profile_start ("mainloop");
+#ifdef ENABLE_PYTHON
+		pyg_begin_allow_threads;
+#endif
 		gtk_main ();
+#ifdef ENABLE_PYTHON
+		pyg_end_allow_threads;
+#endif
 		rb_profile_end ("mainloop");
 
 		rb_debug ("out of toplevel loop");
@@ -362,9 +394,11 @@ dbus_load_uri (const char *filename, DBusGProxy *proxy)
 				G_TYPE_STRING, filename,
 				G_TYPE_BOOLEAN, TRUE,
 				G_TYPE_INVALID,
-				G_TYPE_INVALID))
+				G_TYPE_INVALID)) {
 		g_printerr ("Failed to load %s: %s",
 			    filename, error->message);
+		g_error_free (error);
+	}
 }
 
 static void
@@ -373,3 +407,24 @@ main_shell_weak_ref_cb (gpointer data, GObject *objptr)
 	rb_debug ("caught shell finalization");
 	gtk_main_quit ();
 }
+
+static void
+removable_media_scan_finished (RBShell *shell, gpointer data)
+{
+	load_uri_args ((const char **) remaining_args, (GFunc) local_load_uri, shell);
+}
+
+static void
+local_load_uri (const char *filename, RBShell *shell)
+{
+	GError *error = NULL;
+	rb_debug ("Using load_uri for %s", filename);
+	if (!rb_shell_load_uri (shell, filename, TRUE, &error)) {
+		if (error != NULL) {
+			g_printerr ("Failed to load %s: %s",
+				    filename, error->message);
+			g_error_free (error);
+		}
+	}
+}
+
