@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -34,19 +42,21 @@ G_BEGIN_DECLS
 #define RB_IS_SHELL_PREFERENCES_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SHELL_PREFERENCES))
 #define RB_SHELL_PREFERENCES_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SHELL_PREFERENCES, RBShellPreferencesClass))
 
+typedef struct _RBShellPreferences RBShellPreferences;
+typedef struct _RBShellPreferencesClass RBShellPreferencesClass;
 typedef struct RBShellPreferencesPrivate RBShellPreferencesPrivate;
 
-typedef struct
+struct _RBShellPreferences
 {
 	GtkDialog parent;
 
 	RBShellPreferencesPrivate *priv;
-} RBShellPreferences;
+};
 
-typedef struct
+struct _RBShellPreferencesClass
 {
 	GtkDialogClass parent_class;
-} RBShellPreferencesClass;
+};
 
 GType		rb_shell_preferences_get_type		(void);
 
