@@ -1,4 +1,4 @@
-/* 
+/*
  *  arch-tag: Header for search entry/browse toggle container
  *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,19 +44,22 @@ G_BEGIN_DECLS
 #define RB_IS_SOURCE_HEADER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SOURCE_HEADER))
 #define RB_SOURCE_HEADER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SOURCE_HEADER, RBSourceHeaderClass))
 
+typedef struct _RBSourceHeader RBSourceHeader;
+typedef struct _RBSourceHeaderClass RBSourceHeaderClass;
+
 typedef struct RBSourceHeaderPrivate RBSourceHeaderPrivate;
 
-typedef struct
+struct _RBSourceHeader
 {
 	GtkTable parent;
 
 	RBSourceHeaderPrivate *priv;
-} RBSourceHeader;
+};
 
-typedef struct
+struct _RBSourceHeaderClass
 {
 	GtkTableClass parent_class;
-} RBSourceHeaderClass;
+};
 
 GType			rb_source_header_get_type	(void);
 
@@ -56,9 +67,7 @@ RBSourceHeader *	rb_source_header_new		(GtkUIManager   *mgr,
                                                          GtkActionGroup *actiongroup);
 
 void			rb_source_header_set_source	(RBSourceHeader *header,
-							 RBSource *player);
-
-RBSource *		rb_source_header_get_source	(RBSourceHeader *shell_player);
+							 RBSource *source);
 
 void			rb_source_header_clear_search	(RBSourceHeader *header);
 void			rb_source_header_sync_control_state (RBSourceHeader *header);
