@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -37,29 +45,32 @@ G_BEGIN_DECLS
 #define RB_IS_STATUSBAR_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_STATUSBAR))
 #define RB_STATUSBAR_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_STATUSBAR, RBStatusbarClass))
 
+typedef struct _RBStatusbar RBStatusbar;
+typedef struct _RBStatusbarClass RBStatusbarClass;
+
 typedef struct RBStatusbarPrivate RBStatusbarPrivate;
 
-typedef struct
+struct _RBStatusbar
 {
 	GtkStatusbar parent;
 
 	RBStatusbarPrivate *priv;
-} RBStatusbar;
+};
 
-typedef struct
+struct _RBStatusbarClass
 {
 	GtkStatusbarClass parent_class;
-} RBStatusbarClass;
+};
 
 GType			rb_statusbar_get_type	(void);
 
 RBStatusbar *		rb_statusbar_new	(RhythmDB *db,
-						 GtkActionGroup *actiongroup);
+						 GtkUIManager *ui_manager);
 
-void			rb_statusbar_set_source	(RBStatusbar *bar,
-						 RBSource *player);
+void			rb_statusbar_set_source	(RBStatusbar *statusbar,
+						 RBSource *source);
 
-void			rb_statusbar_set_progress(RBStatusbar *bar, 
+void			rb_statusbar_set_progress(RBStatusbar *statusbar,
 						  double progress,
 						  const char *text);
 
