@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,6 +28,41 @@
  *
  */
 
+/**
+ * SECTION:rb-entry-view
+ * @short_description: a #GtkTreeView for displaying track listings
+ *
+ * This class provides a predefined set of columns for displaying the
+ * common set of #RhythmDBEntry properties, but also allows custom columns
+ * to be appended.  The 'playing' column is always created as the first
+ * column in the tree view, displaying a playing or paused image next to
+ * the currently playing entry, and also an error image next to entries for
+ * which a playback error message has been set.  Clicking on the error
+ * image opens a dialog displaying the full message.
+ *
+ * All columns added to entry view columns should be expandable, or have a fixed
+ * minimum width set.  Otherwise, the tree view must measure the contents of each
+ * row to assign sizes, which is very slow for large track lists.  All the predefined
+ * column types handle this correctly.
+ *
+ * The set of visible columns is controlled by a (single) GConf key, which
+ * contains a comma-delimited list of property names for which the associated
+ * columns are visible.  The entry view object tracks this automatically.
+ *
+ * If a GConf key for sorting is provided when constructing the entry view,
+ * it is watched and updated.  The sort settings consist of a column name
+ * and an order (ascending or descending).
+ */
+
+/**
+ * RBEntryViewColumn:
+ *
+ * Predefined column types to use in #RBEntryView<!-- -->s.  Use
+ * #rb_entry_view_append_column to add these to an entry view.
+ * The predefined column names map directly to the #RhythmDBEntry properties
+ * the columns display.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -238,6 +281,11 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 
 	widget_class->grab_focus = rb_entry_view_grab_focus;
 
+	/**
+	 * RBEntryView:db:
+	 *
+	 * #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -245,6 +293,11 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							      "RhythmDB database",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBEntryView:shell-player:
+	 *
+	 * #RBShellPlayer instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SHELL_PLAYER,
 					 g_param_spec_object ("shell-player",
@@ -252,6 +305,11 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							      "RBShellPlayer object",
 							      RB_TYPE_SHELL_PLAYER,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBEntryView:model:
+	 *
+	 * The #RhythmDBQueryModel backing the view
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_MODEL,
 					 g_param_spec_object ("model",
@@ -259,13 +317,23 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							      "RhythmDBQueryModel",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
+	/**
+	 * RBEntryView:sort-key:
+	 *
+	 * The GConf key that controls the sort order for the view
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SORTING_KEY,
 					 g_param_spec_string ("sort-key",
 							      "sorting key",
 							      "sorting key",
 							      "",
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE));
+	/**
+	 * RBEntryView:is-drag-source:
+	 *
+	 * If TRUE, the view acts as a data source for drag and drop operations.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_IS_DRAG_SOURCE,
 					 g_param_spec_boolean ("is-drag-source",
@@ -273,6 +341,11 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							       "whether or not this is a drag source",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBEntryView:is-drag-dest:
+	 *
+	 * If TRUE, the view acts as a destination for drag and drop operations.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_IS_DRAG_DEST,
 					 g_param_spec_boolean ("is-drag-dest",
@@ -280,6 +353,12 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							       "whether or not this is a drag dest",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBEntryView:playing-state:
+	 *
+	 * Determines the icon to show in the 'playing' column next to the current
+	 * playing entry.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_PLAYING_STATE,
 					 g_param_spec_int ("playing-state",
@@ -289,6 +368,13 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 							   RB_ENTRY_VIEW_PAUSED,
 							   RB_ENTRY_VIEW_NOT_PLAYING,
 							   G_PARAM_READWRITE));
+	/**
+	 * RBEntryView::entry-added
+	 * @view: the #RBEntryView
+	 * @entry: the #RhythmDBEntry that was added
+	 *
+	 * Emitted when an entry is added to the view
+	 */
 	rb_entry_view_signals[ENTRY_ADDED] =
 		g_signal_new ("entry-added",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -299,6 +385,13 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      RHYTHMDB_TYPE_ENTRY);
+	/**
+	 * RBEntryView::entry-deleted:
+	 * @view: the #RBEntryView
+	 * @entry: the #RhythmDBEntry that was removed
+	 *
+	 * Emitted when an entry has been removed from the view
+	 */
 	rb_entry_view_signals[ENTRY_DELETED] =
 		g_signal_new ("entry-deleted",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -309,6 +402,12 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      RHYTHMDB_TYPE_ENTRY);
+	/**
+	 * RBEntryView::entries-replaced:
+	 * @view: the #RBEntryView
+	 *
+	 * Emitted when the model backing the entry view is replaced.
+	 */
 	rb_entry_view_signals[ENTRIES_REPLACED] =
 		g_signal_new ("entries-replaced",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -318,6 +417,14 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	/**
+	 * RBEntryView::entry-activated:
+	 * @view: the #RBEntryView
+	 * @entry: the #RhythmDBEntry that was activated
+	 *
+	 * Emitted when an entry in the view is activated (by double clicking
+	 * or by various key presses)
+	 */
 	rb_entry_view_signals[ENTRY_ACTIVATED] =
 		g_signal_new ("entry-activated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -328,6 +435,12 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      RHYTHMDB_TYPE_ENTRY);
+	/**
+	 * RBEntryView::selection-changed:
+	 * @view: the #RBEntryView
+	 *
+	 * Emitted when the set of selected entries changes
+	 */
 	rb_entry_view_signals[SELECTION_CHANGED] =
 		g_signal_new ("selection-changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -337,6 +450,18 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	/**
+	 * RBEntryView::show-popup
+	 * @view: the #RBEntryView
+	 * @over_entry: if TRUE, the popup request was made while pointing
+	 * at an entry in the view
+	 *
+	 * Emitted when the user performs an action that should result in a
+	 * popup menu appearing.  If the action was a mouse button click, 
+	 * over_entry is FALSE if the mouse pointer was in the blank space after
+	 * the last row in the view.  If the action was a key press, over_entry
+	 * is FALSE if no rows in the view are selected.
+	 */
 	rb_entry_view_signals[SHOW_POPUP] =
 		g_signal_new ("show_popup",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -347,6 +472,14 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
+	/**
+	 * RBEntryView::have-selection-changed:
+	 * @view: the #RBEntryView
+	 * @have_selection: TRUE if one or more rows are selected
+	 *
+	 * Emitted when the user first selects a row, or when no rows are selected
+	 * any more.
+	 */
 	rb_entry_view_signals[HAVE_SEL_CHANGED] =
 		g_signal_new ("have_selection_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -357,6 +490,12 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
+	/**
+	 * RBEntryView::sort-order-changed:
+	 * @view: the #RBEntryView
+	 *
+	 * Emitted when the user changes the sort order for the view
+	 */
 	rb_entry_view_signals[SORT_ORDER_CHANGED] =
 		g_signal_new ("sort-order-changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -573,6 +712,7 @@ rb_entry_view_set_property (GObject *object,
 			    const GValue *value,
 			    GParamSpec *pspec)
 {
+	char *old_sorting_key;
 	RBEntryView *view = RB_ENTRY_VIEW (object);
 
 	switch (prop_id) {
@@ -583,8 +723,42 @@ rb_entry_view_set_property (GObject *object,
 		rb_entry_view_set_shell_player_internal (view, g_value_get_object (value));
 		break;
 	case PROP_SORTING_KEY:
-		g_free (view->priv->sorting_key);
+		/* Remove notification on old key, if any. */
+		if (view->priv->sorting_key) {
+			eel_gconf_notification_remove (view->priv->sorting_gconf_notification_id);
+			view->priv->sorting_gconf_notification_id = 0;
+		}
+
+		old_sorting_key = view->priv->sorting_key;
 		view->priv->sorting_key = g_value_dup_string (value);
+
+		if (view->priv->sorting_key && view->priv->sorting_key[0] != '\0') {
+			char *new_sorting_type;
+
+			/* Set up notification on the new key. */
+			view->priv->sorting_gconf_notification_id =
+				eel_gconf_notification_add (view->priv->sorting_key,
+							    rb_entry_view_sort_key_changed_cb,
+							    view);
+
+			rb_entry_view_set_columns_clickable (view, TRUE);
+
+			/* If there was already a sorting key, assume this is a rename. */
+			if (old_sorting_key && old_sorting_key[0] != '\0') {
+				/* Propagate existing sort order into the new GConf key. */
+				eel_gconf_set_string (view->priv->sorting_key,
+						      rb_entry_view_get_sorting_type (view));
+
+				eel_gconf_unset (old_sorting_key);
+			}
+
+			/* Synchronise UI sorting order with the new key. */
+			new_sorting_type = eel_gconf_get_string (view->priv->sorting_key);
+			rb_entry_view_set_sorting_type (view, new_sorting_type);
+			g_free (new_sorting_type);
+		}
+
+		g_free (old_sorting_key);
 		break;
 	case PROP_MODEL:
 		rb_entry_view_set_model_internal (view, g_value_get_object (value));
@@ -645,6 +819,23 @@ rb_entry_view_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_entry_view_new:
+ * @db: the #RhythmDB instance
+ * @shell_player: the #RBShellPlayer instance
+ * @sort_key: the GConf key controlling the sort order for the view
+ * @is_drag_source: if TRUE, the view should act as a drag and drop data source
+ * @is_drag_dest: if TRUE, the view should act as a drag and drop destination
+ *
+ * Creates a new entry view.  If it makes sense to allow the user to drag entries
+ * from this entry view to other sources, @is_drag_source should be TRUE.  If it
+ * makes sense to allow the user to drag entries from other sources to this view,
+ * @is_drag_dest should be TRUE.  Drag and drop in this sense is used for two purposes:
+ * to transfer tracks between the filesystem and removable devices, and to add tracks
+ * to playlists.
+ *
+ * Return value: the new entry view
+ */
 RBEntryView *
 rb_entry_view_new (RhythmDB *db,
 		   GObject *shell_player,
@@ -658,7 +849,7 @@ rb_entry_view_new (RhythmDB *db,
 					   "hadjustment", NULL,
 					   "vadjustment", NULL,
 					   "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
-					   "vscrollbar_policy", GTK_POLICY_ALWAYS,
+					   "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
 					   "shadow_type", GTK_SHADOW_IN,
 					   "db", db,
 					   "shell-player", RB_SHELL_PLAYER (shell_player),
@@ -672,6 +863,13 @@ rb_entry_view_new (RhythmDB *db,
 	return view;
 }
 
+/**
+ * rb_entry_view_set_model:
+ * @view: the #RBEntryView
+ * @model: the new #RhythmDBQueryModel to use for the view
+ *
+ * Replaces the model backing the entry view.
+ */
 void
 rb_entry_view_set_model (RBEntryView *view,
 			 RhythmDBQueryModel *model)
@@ -945,7 +1143,17 @@ rb_entry_view_sync_sorting (RBEntryView *view)
 	g_free (column_name);
 }
 
-const char *
+/**
+ * rb_entry_view_get_sorting_type:
+ * @view: an #RBEntryView
+ *
+ * Constructs a string that describes the sort settings for the entry view.
+ * This consists of a column name and an order ('ascending' or 'descending')
+ * separated by a comma.
+ *
+ * Return value: sort order description (must be freed)
+ */
+char *
 rb_entry_view_get_sorting_type (RBEntryView *view)
 {
 	char *sorttype;
@@ -965,12 +1173,21 @@ rb_entry_view_get_sorting_type (RBEntryView *view)
 		g_assert_not_reached ();
 	}
 
-	sorttype = g_strdup(key->str);
-	g_string_free (key, TRUE);
+	sorttype = key->str;
+	g_string_free (key, FALSE);
 
 	return sorttype;
 }
 
+/**
+ * rb_entry_view_set_sorting_type:
+ * @view: a #RBEntryView
+ * @sorttype: sort order description
+ *
+ * Changes the sort order for the entry view.  The sort order
+ * description must be a column name, followed by a comma, followed
+ * by an order description ('ascending' or 'descending').
+ */
 void
 rb_entry_view_set_sorting_type (RBEntryView *view,
 				const char *sorttype)
@@ -1001,6 +1218,14 @@ rb_entry_view_set_sorting_type (RBEntryView *view,
 	rb_entry_view_sync_sorting (view);
 }
 
+/**
+ * rb_entry_view_get_sorting_order:
+ * @view: a #RBEntryView
+ * @column_name: returns the sort column name
+ * @sort_order: returns the sort ordering as a #GtkSortType value
+ *
+ * Retrieves the sort settings for the view.
+ */
 void
 rb_entry_view_get_sorting_order (RBEntryView *view,
 				 char **column_name,
@@ -1017,6 +1242,14 @@ rb_entry_view_get_sorting_order (RBEntryView *view,
 	}
 }
 
+/**
+ * rb_entry_view_set_sorting_order:
+ * @view: a #RBEntryView
+ * @column_name: name of the column to sort on
+ * @sort_order: order to sort in, as a #GtkSortOrder
+ *
+ * Sets the sort order for the entry view.
+ */
 void
 rb_entry_view_set_sorting_order (RBEntryView *view,
 				 const char *column_name,
@@ -1058,6 +1291,90 @@ rb_entry_view_column_clicked_cb (GtkTreeViewColumn *column, RBEntryView *view)
 		eel_gconf_set_string (view->priv->sorting_key, rb_entry_view_get_sorting_type(view));
 }
 
+/**
+ * rb_entry_view_get_column:
+ * @view: a #RBEntryView
+ * @coltype: type of column to retrieve
+ *
+ * Retrieves a predefined column from the entry view.  This can be used
+ * to insert additional cell renderers into the column.
+ *
+ * Return value: a #GtkTreeViewColumn instance, or NULL
+ */
+GtkTreeViewColumn *
+rb_entry_view_get_column (RBEntryView *view, RBEntryViewColumn coltype)
+{
+	RhythmDBPropType propid;
+
+	/* convert column type to property ID */
+	switch (coltype) {
+	case RB_ENTRY_VIEW_COL_TRACK_NUMBER:
+		propid = RHYTHMDB_PROP_TRACK_NUMBER;
+		break;
+	case RB_ENTRY_VIEW_COL_TITLE:
+		propid = RHYTHMDB_PROP_TITLE;
+		break;
+	case RB_ENTRY_VIEW_COL_ARTIST:
+		propid = RHYTHMDB_PROP_ARTIST;
+		break;
+	case RB_ENTRY_VIEW_COL_ALBUM:
+		propid = RHYTHMDB_PROP_ALBUM;
+		break;
+	case RB_ENTRY_VIEW_COL_GENRE:
+		propid = RHYTHMDB_PROP_GENRE;
+		break;
+	case RB_ENTRY_VIEW_COL_DURATION:
+		propid = RHYTHMDB_PROP_DURATION;
+		break;
+	case RB_ENTRY_VIEW_COL_YEAR:
+		propid = RHYTHMDB_PROP_DATE;
+		break;
+	case RB_ENTRY_VIEW_COL_QUALITY:
+		propid = RHYTHMDB_PROP_BITRATE;
+		break;
+	case RB_ENTRY_VIEW_COL_RATING:
+		propid = RHYTHMDB_PROP_RATING;
+		break;
+	case RB_ENTRY_VIEW_COL_PLAY_COUNT:
+		propid = RHYTHMDB_PROP_PLAY_COUNT;
+		break;
+	case RB_ENTRY_VIEW_COL_LAST_PLAYED:
+		propid = RHYTHMDB_PROP_LAST_PLAYED;
+		break;
+	case RB_ENTRY_VIEW_COL_FIRST_SEEN:
+		propid = RHYTHMDB_PROP_FIRST_SEEN;
+		break;
+	case RB_ENTRY_VIEW_COL_LAST_SEEN:
+		propid = RHYTHMDB_PROP_LAST_SEEN;
+		break;
+	case RB_ENTRY_VIEW_COL_LOCATION:
+		propid = RHYTHMDB_PROP_LOCATION;
+		break;
+	case RB_ENTRY_VIEW_COL_ERROR:
+		propid = RHYTHMDB_PROP_PLAYBACK_ERROR;
+		break;
+	default:
+		g_assert_not_reached ();
+		propid = -1;
+		break;
+	}
+
+	/* find the column */
+	return (GtkTreeViewColumn *)g_hash_table_lookup (view->priv->propid_column_map, GINT_TO_POINTER (propid));
+}
+
+/**
+ * rb_entry_view_append_column:
+ * @view: a #RBEntryView
+ * @coltype: type of column to append
+ * @always_visible: if TRUE, ignore the user's column visibility settings
+ *
+ * Appends a predefined column type to the set of columns already present
+ * in the entry view.  If @always_visible is TRUE, the column will ignore
+ * the user's coulmn visibility settings and will always be visible.
+ * This should only be used when it is vital for the purpose of the 
+ * source that the column be visible.
+ */
 void
 rb_entry_view_append_column (RBEntryView *view,
 			     RBEntryViewColumn coltype,
@@ -1304,6 +1621,18 @@ rb_entry_view_append_column (RBEntryView *view,
 	rb_entry_view_append_column_custom (view, column, title, key, sort_func, GINT_TO_POINTER (sort_propid), NULL);
 }
 
+/**
+ * rb_entry_view_append_column_custom:
+ * @view: a #RBEntryView
+ * @column: a #GtkTreeViewColumn to append
+ * @title: title for the column (translated)
+ * @key: sort key for the column (not translated)
+ * @sort_func: comparison function to use for sorting on the column
+ * @data: data to pass to the sort function
+ * @data_destroy: function to use to destroy the sort data
+ *
+ * Appends a custom column to the entry view.  
+ */
 void
 rb_entry_view_append_column_custom (RBEntryView *view,
 				    GtkTreeViewColumn *column,
@@ -1316,6 +1645,19 @@ rb_entry_view_append_column_custom (RBEntryView *view,
 	rb_entry_view_insert_column_custom (view, column, title, key, sort_func, data, data_destroy, -1);
 }
 
+/**
+ * rb_entry_view_insert_column_custom:
+ * @view: a #RBEntryView
+ * @column: a #GtkTreeViewColumn to append
+ * @title: title for the column (translated)
+ * @key: sort key for the column (not translated)
+ * @sort_func: comparison function to use for sorting on the column
+ * @data: data to pass to the sort function
+ * @data_destroy: function to use to destroy the sort data
+ * @position: position at which to insert the column (-1 to insert at the end)
+ *
+ * Inserts a custom column at the specified position.
+ */
 void
 rb_entry_view_insert_column_custom (RBEntryView *view,
 				    GtkTreeViewColumn *column,
@@ -1355,6 +1697,14 @@ rb_entry_view_insert_column_custom (RBEntryView *view,
 	rb_entry_view_sync_sorting (view);
 }
 
+/**
+ * rb_entry_view_set_columns_clickable:
+ * @view: a #RBEntryView
+ * @clickable: if TRUE, sortable columns will be made clickable
+ *
+ * Makes the headers for sortable columns (those for which a sort function was
+ * provided) clickable, so the user can set the sort order.
+ */
 void
 rb_entry_view_set_columns_clickable (RBEntryView *view,
 				     gboolean clickable)
@@ -1475,18 +1825,6 @@ rb_entry_view_constructor (GType type,
 		eel_gconf_notification_add (CONF_UI_COLUMNS_SETUP,
 					    rb_entry_view_columns_config_changed_cb,
 					    view);
-	if (view->priv->sorting_key) {
-		view->priv->sorting_gconf_notification_id =
-			eel_gconf_notification_add (view->priv->sorting_key,
-						    rb_entry_view_sort_key_changed_cb,
-						    view);
-	}
-
-	if (view->priv->sorting_key) {
-		char *s = eel_gconf_get_string (view->priv->sorting_key);
-		rb_entry_view_set_sorting_type (view, s);
-		g_free (s);
-	}
 
 	{
 		RhythmDBQueryModel *query_model;
@@ -1600,6 +1938,13 @@ harvest_entries (GtkTreeModel *model,
 	return FALSE;
 }
 
+/**
+ * rb_entry_view_get_selected_entries:
+ * @view: a #RBEntryView
+ *
+ * Return value: a #GList containing the currently selected entries in the view
+ *  (must be freed)
+ */
 GList *
 rb_entry_view_get_selected_entries (RBEntryView *view)
 {
@@ -1690,12 +2035,24 @@ rb_entry_view_selection_changed_cb (GtkTreeSelection *selection,
 		view->priv->selection_changed_id = g_idle_add ((GSourceFunc)rb_entry_view_emit_selection_changed, view);
 }
 
+/**
+ * rb_entry_view_have_selection:
+ * @view: a #RBEntryView
+ *
+ * Return value: TRUE if one or more rows are selected in the view
+ */
 gboolean
 rb_entry_view_have_selection (RBEntryView *view)
 {
 	return view->priv->have_selection;
 }
 
+/**
+ * rb_entry_view_have_complete_selection:
+ * @view: a #RBEntryView
+ *
+ * Return value: TRUE if all rows in the view are selected
+ */
 gboolean
 rb_entry_view_have_complete_selection (RBEntryView *view)
 {
@@ -1800,18 +2157,38 @@ rb_entry_view_rows_reordered_cb (GtkTreeModel *model,
 	gtk_widget_queue_draw (GTK_WIDGET (view));
 }
 
+/**
+ * rb_entry_view_select_all:
+ * @view: a #RBEntryView
+ *
+ * Selects all rows in the view
+ */
 void
 rb_entry_view_select_all (RBEntryView *view)
 {
 	gtk_tree_selection_select_all (view->priv->selection);
 }
 
+/**
+ * rb_entry_view_select_none:
+ * @view: a #RBEntryView
+ *
+ * Deselects all rows in the view.
+ */
 void
 rb_entry_view_select_none (RBEntryView *view)
 {
 	gtk_tree_selection_unselect_all (view->priv->selection);
 }
 
+/**
+ * rb_entry_view_select_entry:
+ * @view: a #RBEntryView
+ * @entry: a #RhythmDBEntry to select
+ *
+ * If the specified entry is present in the view, it is added
+ * to the selection.
+ */
 void
 rb_entry_view_select_entry (RBEntryView *view,
 			    RhythmDBEntry *entry)
@@ -1829,6 +2206,14 @@ rb_entry_view_select_entry (RBEntryView *view,
 	}
 }
 
+/**
+ * rb_entry_view_scroll_to_entry:
+ * @view: a #RBEntryView
+ * @entry: a #RhythmDBEntry to scroll to
+ *
+ * If the specified entry is present in the view, the view will be
+ * scrolled so that the entry is visible.
+ */
 void
 rb_entry_view_scroll_to_entry (RBEntryView *view,
 			       RhythmDBEntry *entry)
@@ -1864,6 +2249,13 @@ rb_entry_view_scroll_to_iter (RBEntryView *view,
 	gtk_tree_path_free (path);
 }
 
+/**
+ * rb_entry_view_get_entry_visible:
+ * @view: a #RBEntryView
+ * @entry: a #RhythmDBEntry to check
+ *
+ * Return value: TRUE if the entry is present in the view and is currently visible
+ */
 gboolean
 rb_entry_view_get_entry_visible (RBEntryView *view,
 				 RhythmDBEntry *entry)
@@ -1879,6 +2271,13 @@ rb_entry_view_get_entry_visible (RBEntryView *view,
 	return realized && visible;
 }
 
+/**
+ * rb_entry_view_get_entry_contained:
+ * @view: a #RBEntryView
+ * @entry: a #RhythmDBEntry to check
+ *
+ * Return value: TRUE if the entry is present in the view
+ */
 gboolean
 rb_entry_view_get_entry_contained (RBEntryView *view,
 				   RhythmDBEntry *entry)
@@ -1924,6 +2323,15 @@ rb_entry_view_entry_is_visible (RBEntryView *view,
 	*visible = (rect.y != 0 && rect.height != 0);
 }
 
+/**
+ * rb_entry_view_enable_drag_source:
+ * @view: a #RBEntryView
+ * @targets: an array of #GtkTargetEntry structures defining the drag data targets
+ * @n_targets: the number of entries in the target array
+ *
+ * Enables the entry view to act as a data source for drag an drop operations,
+ * using a specified set of data targets.
+ */
 void
 rb_entry_view_enable_drag_source (RBEntryView *view,
 				 const GtkTargetEntry *targets,
@@ -2021,6 +2429,16 @@ rb_entry_view_sync_columns_visible (RBEntryView *view)
 	g_free (config);
 }
 
+/**
+ * rb_entry_view_set_state:
+ * @view: a #RBEntryView
+ * @state: the new playing entry state
+ *
+ * Sets the icon to be drawn in the 'playing' column next to the
+ * current playing entry.  RB_ENTRY_VIEW_PLAYING and RB_ENTRY_VIEW_PAUSED
+ * should be used when the source containing the entry view is playing,
+ * and RB_ENTRY_VIEW_NOT_PLAYING otherwise.
+ */
 void
 rb_entry_view_set_state (RBEntryView *view,
 			 RBEntryViewState state)
@@ -2055,50 +2473,43 @@ rb_entry_view_emit_row_changed (RBEntryView *view,
 	return TRUE;
 }
 
+/**
+ * rb_entry_view_set_fixed_column_width:
+ * @view: a #RBEntryView
+ * @column: the column to set the width for
+ * @renderer: a temporary cell renderer to use
+ * @strings: a NULL-terminated array of strings that will be displayed in the column
+ *
+ * Helper function for calling @rb_set_tree_view_column_fixed_width on
+ * a column.  This is important for performance reasons, as having the
+ * tree view measure the strings in each of 20000 rows is very slow.
+ */
 void
 rb_entry_view_set_fixed_column_width (RBEntryView *view,
 				      GtkTreeViewColumn *column,
 				      GtkCellRenderer *renderer,
 				      const gchar **strings)
 {
-	gint max_width = 0;
-	int i = 0;
-
-	while (strings[i] != NULL) {
-		gint width;
-		g_object_set (renderer, "text", strings[i], NULL);
-		gtk_cell_renderer_get_size (renderer,
-					    view->priv->treeview,
-					    NULL,
-					    NULL, NULL,
-					    &width, NULL);
-
-		if (width > max_width)
-			max_width = width;
-
-		i++;
-	}
-
-	/* include some arbitrary amount of padding, just to be safeish */
-	gtk_tree_view_column_set_fixed_width (column, max_width + 5);
+	rb_set_tree_view_column_fixed_width (view->priv->treeview,
+					     column,
+					     renderer,
+					     strings,
+					     5);
 }
 
+/**
+ * rb_entry_view_get_time_date_column_sample:
+ *
+ * Returns a sample string for use in columns displaying times
+ * and dates in 'friendly' form (see @rb_utf_friendly_time).
+ * For use with @rb_entry_view_set_fixed_column_width.
+ *
+ * Return value: sample date string
+ */
 const char *
 rb_entry_view_get_time_date_column_sample ()
 {
 	static const char *sample = NULL;
-	/*
-	 * Currently, Japanese is the only translation that uses
-	 * anything other than %Y, %m ,%d, %H, and %M to format dates.
-	 * It uses %B (month name) and %e (days), and the values for
-	 * the month name appear to all consist of the month number
-	 * followed by a single character, so they're of consistent
-	 * width.  So, this approach should work for every locale.
-	 *
-	 * Midnight on September 30th, 2000 is the widest date/time I
-	 * can think of.  2000-09-30 00:00.
-	 */
-
 	if (sample == NULL) {
  		time_t then;
 
@@ -2111,6 +2522,13 @@ rb_entry_view_get_time_date_column_sample ()
 	return sample;
 }
 
+/**
+ * rb_entry_view_resort_model:
+ * @view: a #RBEntryView to resort
+ *
+ * Resorts the entries in the entry view.  Mostly to be used
+ * when a new model is associated with the view.
+ */
 void
 rb_entry_view_resort_model (RBEntryView *view)
 {
