@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -38,6 +46,20 @@
 #include "rb-util.h"
 #include "rhythmdb.h"
 
+/**
+ * SECTION:rb-header
+ * @short_description: playback area widgetry
+ *
+ * The RBHeader widget displays information about the current playing track
+ * (title, album, artist), the elapsed or remaining playback time, and a
+ * position slider indicating the playback position.  It translates slider
+ * move and drag events into seek requests for the player backend.
+ *
+ * For shoutcast-style streams, the title/artist/album display is supplemented
+ * by metadata extracted from the stream.  See #RBStreamingSource for more information
+ * on how the metadata is reported.
+ */
+
 static void rb_header_class_init (RBHeaderClass *klass);
 static void rb_header_init (RBHeader *header);
 static void rb_header_finalize (GObject *object);
@@ -52,6 +74,7 @@ static void rb_header_get_property (GObject *object,
 static void rb_header_set_show_timeline (RBHeader *header,
 			                 gboolean show);
 static void rb_header_update_elapsed (RBHeader *header);
+static void apply_slider_position (RBHeader *header);
 static gboolean slider_press_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event, RBHeader *header);
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
@@ -79,7 +102,6 @@ struct RBHeaderPrivate
 	gboolean slider_locked;
 	guint slider_moved_timeout;
 	long latest_set_time;
-	guint value_changed_update_handler;
 	GtkWidget *elapsed;
 
 	guint elapsed_time;
@@ -94,6 +116,7 @@ enum
 	PROP_SHELL_PLAYER,
 	PROP_ENTRY,
 	PROP_SEEKABLE,
+	PROP_SLIDER_DRAGGING
 };
 
 #define TITLE_MARKUP(xTITLE) g_markup_printf_escaped ("<big><b>%s</b></big>", xTITLE)
@@ -113,6 +136,11 @@ rb_header_class_init (RBHeaderClass *klass)
 	object_class->set_property = rb_header_set_property;
 	object_class->get_property = rb_header_get_property;
 
+	/**
+	 * RBHeader:db:
+	 *
+	 * #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -121,6 +149,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE));
 
+	/**
+	 * RBHeader:entry:
+	 *
+	 * The #RhythmDBEntry to display
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY,
 					 g_param_spec_boxed ("entry",
@@ -128,6 +161,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							     "RhythmDBEntry pointer",
 							     RHYTHMDB_TYPE_ENTRY,
 							     G_PARAM_READWRITE));
+	/**
+	 * RBHeader:shell-player:
+	 *
+	 * The #RBShellPlayer instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SHELL_PLAYER,
 					 g_param_spec_object ("shell-player",
@@ -135,6 +173,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							      "RBShellPlayer object",
 							      RB_TYPE_SHELL_PLAYER,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBHeader:seekable:
+	 *
+	 * If TRUE, the header should allow seeking by dragging the playback position slider
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SEEKABLE,
 					 g_param_spec_boolean ("seekable",
@@ -143,13 +186,26 @@ rb_header_class_init (RBHeaderClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * RBHeader:slider-dragging:
+	 *
+	 * Whether the song position slider is currently being dragged.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_SLIDER_DRAGGING,
+					 g_param_spec_boolean ("slider-dragging",
+						 	       "slider dragging",
+							       "slider dragging",
+							       FALSE,
+							       G_PARAM_READABLE));
+
 	g_type_class_add_private (klass, sizeof (RBHeaderPrivate));
 }
 
 static void
 rb_header_init (RBHeader *header)
 {
-	/**
+	/*
 	 * The children in this widget look like this:
 	 * RBHeader
 	 *   GtkHBox
@@ -234,6 +290,24 @@ rb_header_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_header_parent_class)->finalize (object);
 }
 
+static void
+rb_header_set_playing_entry_internal (RBHeader *header, RhythmDBEntry *entry)
+{
+	if (header->priv->entry == entry)
+		return;
+
+	header->priv->entry = entry;
+	if (header->priv->entry) {
+		header->priv->duration = rhythmdb_entry_get_ulong (header->priv->entry,
+								   RHYTHMDB_PROP_DURATION);
+	} else {
+		header->priv->duration = 0;
+	}
+
+	header->priv->adjustment->upper = header->priv->duration;
+	gtk_adjustment_changed (header->priv->adjustment);
+}
+
 static void
 rb_header_set_property (GObject *object,
 			guint prop_id,
@@ -247,13 +321,7 @@ rb_header_set_property (GObject *object,
 		header->priv->db = g_value_get_object (value);
 		break;
 	case PROP_ENTRY:
-		header->priv->entry = g_value_get_boxed (value);
-		if (header->priv->entry) {
-			header->priv->duration = rhythmdb_entry_get_ulong (header->priv->entry,
-									   RHYTHMDB_PROP_DURATION);
-		} else {
-			header->priv->duration = 0;
-		}
+		rb_header_set_playing_entry_internal (header, g_value_get_boxed (value));
 		break;
 	case PROP_SHELL_PLAYER:
 		header->priv->shell_player = g_value_get_object (value);
@@ -292,12 +360,22 @@ rb_header_get_property (GObject *object,
 	case PROP_SEEKABLE:
 		g_value_set_boolean (value, header->priv->seekable);
 		break;
+	case PROP_SLIDER_DRAGGING:
+		g_value_set_boolean (value, header->priv->slider_dragging);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+/**
+ * rb_header_new:
+ * @shell_player: the #RBShellPlayer instance
+ * @db: the #RhythmDB instance
+ *
+ * Return value: the header widget
+ */
 RBHeader *
 rb_header_new (RBShellPlayer *shell_player, RhythmDB *db)
 {
@@ -313,6 +391,14 @@ rb_header_new (RBShellPlayer *shell_player, RhythmDB *db)
 	return header;
 }
 
+/**
+ * rb_header_set_playing_entry:
+ * @header: the #RBHeader
+ * @entry: the new playing #RhythmDBEntry
+ * @seekable: whether the new playing entry is seekable
+ *
+ * Sets a new playing entry to be displayed in the header.
+ */
 void
 rb_header_set_playing_entry (RBHeader *header,
 			     RhythmDBEntry *entry,
@@ -347,12 +433,23 @@ get_extra_metadata (RhythmDB *db, RhythmDBEntry *entry, const char *field, char
 	}
 }
 
+/**
+ * rb_header_sync:
+ * @header: the #RBHeader
+ *
+ * Updates the header widget to be consistent with the current playing entry
+ * including all streaming metadata.
+ */
 void
 rb_header_sync (RBHeader *header)
 {
 	char *label_text;
+	const char *location = "<null>";
 
-	rb_debug ("syncing with entry = %p", header->priv->entry);
+	if (header->priv->entry != NULL) {
+		location = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_LOCATION);
+	}
+	rb_debug ("syncing with entry = %s", location);
 
 	if (header->priv->entry != NULL) {
 		const char *title;
@@ -400,6 +497,11 @@ rb_header_sync (RBHeader *header)
 
 		label_str = g_string_sized_new (100);
 
+		/* stick a right-to-left mark in the string for RTL display */
+		if (gtk_widget_get_direction (GTK_WIDGET (header->priv->song)) == GTK_TEXT_DIR_RTL) {
+			g_string_append (label_str, "\xE2\x80\x8F");
+		}
+
 		append_and_free (label_str, TITLE_MARKUP (title));
 
 		if (artist != NULL && artist[0] != '\0')
@@ -439,6 +541,14 @@ rb_header_sync (RBHeader *header)
 	}
 }
 
+/**
+ * rb_header_set_show_position_slider:
+ * @header: the #RBHeader
+ * @show: whether the position slider should be shown
+ *
+ * Sets the visibility of the position slider.  This is not currently
+ * used properly.
+ */
 void
 rb_header_set_show_position_slider (RBHeader *header,
 				    gboolean show)
@@ -463,30 +573,33 @@ rb_header_set_show_timeline (RBHeader *header,
 	gtk_widget_set_sensitive (header->priv->scaleline, show);
 }
 
-gboolean
+/**
+ * rb_header_sync_time:
+ * @header: the #RBHeader
+ *
+ * Updates the time display components of the header.
+ * If the position slider is being dragged, the display is not updated.
+ * If the duration of the playing entry is known, the position slider is
+ * updated along with the elapsed/remaining time display.  Otherwise,
+ * the slider is made insensitive.
+ */
+void
 rb_header_sync_time (RBHeader *header)
 {
 	guint seconds;
 
 	if (header->priv->shell_player == NULL)
-		return TRUE;
+		return;
 
 	if (header->priv->slider_dragging == TRUE) {
 		rb_debug ("slider is dragging, not syncing");
-		return TRUE;
+		return;
 	}
 
 	seconds = header->priv->elapsed_time;
 
 	if (header->priv->duration > 0) {
-		double progress = 0.0;
-
-		if (seconds > 0) {
-			progress = (double) seconds;
-		} else {
-			header->priv->adjustment->upper = header->priv->duration;
-			g_signal_emit_by_name (G_OBJECT (header->priv->adjustment), "changed");
-		}
+		double progress = (double) seconds;
 
 		header->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (header->priv->adjustment, progress);
@@ -500,8 +613,6 @@ rb_header_sync_time (RBHeader *header)
 	}
 
 	rb_header_update_elapsed (header);
-
-	return TRUE;
 }
 
 static gboolean
@@ -511,24 +622,25 @@ slider_press_callback (GtkWidget *widget,
 {
 	header->priv->slider_dragging = TRUE;
 	header->priv->latest_set_time = -1;
+	g_object_notify (G_OBJECT (header), "slider-dragging");
+
+	/* HACK: we want the behaviour you get with the middle button, so we
+	 * mangle the event.  clicking with other buttons moves the slider in
+	 * step increments, clicking with the middle button moves the slider to
+	 * the location of the click.
+	 */
+	event->button = 2;
+
+
 	return FALSE;
 }
 
 static gboolean
 slider_moved_timeout (RBHeader *header)
 {
-	double progress;
-	long new;
-
 	GDK_THREADS_ENTER ();
 
-	progress = gtk_adjustment_get_value (gtk_range_get_adjustment (GTK_RANGE (header->priv->scale)));
-	new = (long) (progress+0.5);
-
-	rb_debug ("setting time to %ld", new);
-	rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
-
-	header->priv->latest_set_time = new;
+	apply_slider_position (header);
 	header->priv->slider_moved_timeout = 0;
 
 	GDK_THREADS_LEAVE ();
@@ -541,7 +653,6 @@ slider_moved_callback (GtkWidget *widget,
 		       GdkEventMotion *event,
 		       RBHeader *header)
 {
-	GtkAdjustment *adjustment;
 	double progress;
 
 	if (header->priv->slider_dragging == FALSE) {
@@ -549,9 +660,7 @@ slider_moved_callback (GtkWidget *widget,
 		return FALSE;
 	}
 
-	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
-
-	progress = gtk_adjustment_get_value (adjustment);
+	progress = gtk_adjustment_get_value (header->priv->adjustment);
 	header->priv->elapsed_time = (guint) (progress+0.5);
 
 	rb_header_update_elapsed (header);
@@ -567,54 +676,43 @@ slider_moved_callback (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+apply_slider_position (RBHeader *header)
+{
+	double progress;
+	long new;
+
+	progress = gtk_adjustment_get_value (header->priv->adjustment);
+	new = (long) (progress+0.5);
+
+	if (new != header->priv->latest_set_time) {
+		rb_debug ("setting time to %ld", new);
+		rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
+		header->priv->latest_set_time = new;
+	}
+}
+
 static gboolean
 slider_release_callback (GtkWidget *widget,
 			 GdkEventButton *event,
 			 RBHeader *header)
 {
-	double progress;
-	long new;
-	GtkAdjustment *adjustment;
+	/* HACK: see slider_press_callback */
+	event->button = 2;
 
 	if (header->priv->slider_dragging == FALSE) {
 		rb_debug ("slider is not dragging");
 		return FALSE;
 	}
 
-	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
-
-	progress = gtk_adjustment_get_value (adjustment);
-	new = (long) (progress+0.5);
-
-	if (new != header->priv->latest_set_time) {
-		rb_debug ("setting time to %ld", new);
-		rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
-	}
-
-	header->priv->slider_dragging = FALSE;
-
-	rb_header_sync_time (header);
-
 	if (header->priv->slider_moved_timeout != 0) {
 		g_source_remove (header->priv->slider_moved_timeout);
 		header->priv->slider_moved_timeout = 0;
 	}
 
-	return FALSE;
-}
-
-static gboolean
-changed_idle_callback (RBHeader *header)
-{
-	GDK_THREADS_ENTER ();
-
-	slider_release_callback (header->priv->scale, NULL, header);
-
-	header->priv->value_changed_update_handler = 0;
-	rb_debug ("in changed_idle_callback");
-
-	GDK_THREADS_LEAVE ();
-
+	apply_slider_position (header);
+	header->priv->slider_dragging = FALSE;
+	g_object_notify (G_OBJECT (header), "slider-dragging");
 	return FALSE;
 }
 
@@ -622,12 +720,13 @@ static void
 slider_changed_callback (GtkWidget *widget,
 		         RBHeader *header)
 {
+	/* if the slider isn't being dragged, and nothing else is happening,
+	 * this indicates the position was adjusted with a keypress (page up/page down etc.),
+	 * so we should directly apply the change.
+	 */
 	if (header->priv->slider_dragging == FALSE &&
-	    header->priv->slider_locked == FALSE &&
-	    header->priv->value_changed_update_handler == 0) {
-		header->priv->slider_dragging = TRUE;
-		header->priv->value_changed_update_handler =
-			g_idle_add ((GSourceFunc) changed_idle_callback, header);
+	    header->priv->slider_locked == FALSE) {
+		apply_slider_position (header);
 	}
 }
 
