@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -37,23 +45,26 @@ G_BEGIN_DECLS
 #define RB_IS_HEADER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_HEADER))
 #define RB_HEADER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_HEADER, RBHeaderClass))
 
+typedef struct _RBHeader RBHeader;
+typedef struct _RBHeaderClass RBHeaderClass;
+
 typedef struct RBHeaderPrivate RBHeaderPrivate;
 
-typedef struct
+struct _RBHeader
 {
 	GtkHBox parent;
 
 	RBHeaderPrivate *priv;
-} RBHeader;
+};
 
-typedef struct
+struct _RBHeaderClass
 {
 	GtkHBoxClass parent;
-} RBHeaderClass;
+};
 
 GType		rb_header_get_type		(void);
 
-RBHeader *	rb_header_new			(RBShellPlayer *header,
+RBHeader *	rb_header_new			(RBShellPlayer *shell_player,
 						 RhythmDB *db);
 
 void		rb_header_set_playing_entry	(RBHeader *header,
@@ -65,7 +76,7 @@ void		rb_header_set_show_position_slider (RBHeader *header,
 
 void		rb_header_sync			(RBHeader *header);
 
-gboolean	rb_header_sync_time		(RBHeader *header);
+void		rb_header_sync_time		(RBHeader *header);
 
 char *		rb_header_get_elapsed_string	(RBHeader *header);
 
