@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -39,16 +47,19 @@ G_BEGIN_DECLS
 #define RB_IS_PROPERTY_VIEW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_PROPERTY_VIEW))
 #define RB_PROPERTY_VIEW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_PROPERTY_VIEW, RBPropertyViewClass))
 
+typedef struct _RBPropertyView RBPropertyView;
+typedef struct _RBPropertyViewClass RBPropertyViewClass;
+
 typedef struct RBPropertyViewPrivate RBPropertyViewPrivate;
 
-typedef struct
+struct _RBPropertyView
 {
 	GtkScrolledWindow parent;
 
 	RBPropertyViewPrivate *priv;
-} RBPropertyView;
+};
 
-typedef struct
+struct _RBPropertyViewClass
 {
 	GtkScrolledWindowClass parent;
 
@@ -57,7 +68,7 @@ typedef struct
 	void (*property_activated)	(RBPropertyView *view, const char *name);
 	void (*selection_reset)		(RBPropertyView *view);
 	void (*show_popup)		(RBPropertyView *view);
-} RBPropertyViewClass;
+};
 
 GType		rb_property_view_get_type		(void);
 
@@ -72,7 +83,9 @@ void		rb_property_view_set_selection_mode	(RBPropertyView *view,
 void		rb_property_view_reset			(RBPropertyView *view);
 
 void		rb_property_view_set_selection		(RBPropertyView *view,
-							 const GList *names);
+							 const GList *vals);
+
+GList *		rb_property_view_get_selection		(RBPropertyView *view);
 
 RhythmDBPropertyModel * rb_property_view_get_model	(RBPropertyView *view);
 
