@@ -2,12 +2,20 @@
  *  arch-tag: Header for rating renderer object
  *
  *  Copyright (C) 2002 Olivier Martin <oleevye@wanadoo.fr>
- *  
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
- *  
+ *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -15,8 +23,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- * 
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
+ *
  */
 
 #ifndef RB_RATING_H
@@ -32,22 +40,23 @@ G_BEGIN_DECLS
 #define RB_IS_RATING(obj)         (GTK_CHECK_TYPE ((obj), RB_TYPE_RATING))
 #define RB_IS_RATING_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), RB_TYPE_RATING))
 
-typedef struct RBRatingPrivate RBRatingPrivate;
+typedef struct _RBRating RBRating;
+typedef struct _RBRatingClass RBRatingClass;
+typedef struct _RBRatingPrivate RBRatingPrivate;
 
-typedef struct 
+struct _RBRating
 {
 	GtkEventBox parent;
 
 	RBRatingPrivate *priv;
-} RBRating;
+};
 
-typedef struct 
+struct _RBRatingClass
 {
 	GtkEventBoxClass parent;
 
 	void (*rated) (RBRating *rating, double score);
-} RBRatingClass;
-
+};
 
 GtkType    rb_rating_get_type (void);
 
